package com.chinaitop.depot.system.controller;

import com.chinaitop.depot.system.model.ButtonInfo;
import com.chinaitop.depot.system.model.ButtonInfoExample;
import com.chinaitop.depot.system.model.ButtonInfoExample.Criteria;
import com.chinaitop.depot.system.service.ButtonInfoService;
import com.fasterxml.jackson.core.JsonParseException;
import com.fasterxml.jackson.core.type.TypeReference;
import com.fasterxml.jackson.databind.JsonMappingException;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RestController;

import javax.annotation.Resource;
import java.io.IOException;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * 按钮信息控制类
 * @author xueenxi
 *
 */
@RestController
@RequestMapping(value="/buttonInfo")
@Api(description = "按钮信息控制类")
public class ButtonInfoController {
	
	@Resource
	private ButtonInfoService buttonInfoService;
	
	/**
	 * 查找按钮信息
	 * @param pageNum	页码
	 * @param pageSize	每页条数
	 * @return	PageInfo
	 */
	@RequestMapping(value="/getButton", method = RequestMethod.GET)
	@ApiOperation(value="查询按钮信息", notes = "查询按钮信息列表，支持分页")
	@ApiImplicitParams({
			@ApiImplicitParam(name = "pageNum", value = "页码", paramType = "query"),
			@ApiImplicitParam(name = "pageSize", value = "每页条数", paramType = "query"),
			@ApiImplicitParam(name = "funcId", value = "功能编号", paramType = "query")
	})
	public PageInfo<ButtonInfo> getButtonInfo(Integer pageNum, Integer pageSize,
                                              Integer funcId) {
		ButtonInfoExample example = new ButtonInfoExample();
		Criteria criteria = example.createCriteria();
		if (funcId != null) {
			criteria.andFuncIdEqualTo(funcId);
		}
		if (pageNum!=null && pageSize!=null) {
			PageHelper.startPage(pageNum, pageSize);
		}
		List<ButtonInfo> list = buttonInfoService.queryByExample(example);
		PageInfo<ButtonInfo> pageInfo = new PageInfo<ButtonInfo>(list);
		return pageInfo;
	}
	
	/**
	 * 根据编号查找按钮信息
	 * @param id 编号
	 * @return
	 */
	@RequestMapping(value="/edit", method = RequestMethod.GET)
	@ApiOperation(value="获取按钮信息", notes = "根据按钮编号获取按钮信息")
	@ApiImplicitParams({
			@ApiImplicitParam(name = "btnId", value = "按钮编号", paramType = "query")
	})
	public ButtonInfo edit(Integer btnId) {
		ButtonInfo buttonInfo = new ButtonInfo();
		if (btnId != null) {
			buttonInfo = buttonInfoService.findById(btnId);
		}
		return buttonInfo;
	}

	/**
	 * 保存按钮信息
	 * @param buttonInfoJson	按钮信息json串
	 * @return
	 * @throws JsonParseException
	 * @throws JsonMappingException
	 * @throws IOException
	 */
	@SuppressWarnings("unchecked")
	@RequestMapping(value="/save", method = RequestMethod.POST)
	@ApiOperation(value="新增按钮", notes = "新增按钮")
	@ApiImplicitParams({
			@ApiImplicitParam(name = "buttonInfoJson", value = "按钮信息JSON串", paramType = "form")
	})
	public Map<String, Object> save(String buttonInfoJson) throws JsonParseException, JsonMappingException, IOException {
		Map<String, Object> modelMap = new HashMap<>();
		// JSON字符串转对象
		ObjectMapper mapper = new ObjectMapper();
		List<ButtonInfo> buttonInfos = (List<ButtonInfo>)mapper.readValue(buttonInfoJson, new TypeReference<List<ButtonInfo>>(){});
		buttonInfoService.add(buttonInfos);
		modelMap.put("status", "success");
		return modelMap;
	}
	
	/**
	 * 根据编号删除按钮信息
	 * @param btnId	按钮编号
	 * @return
	 * @throws JsonParseException
	 * @throws JsonMappingException
	 * @throws IOException
	 */
	@RequestMapping(value="/remove", method = RequestMethod.POST)
	@ApiOperation(value="删除按钮", notes = "根据按钮编号删除按钮")
	@ApiImplicitParams({
			@ApiImplicitParam(name = "btnId", value = "按钮编号", paramType = "form")
	})
	public Map<String, Object> remove(Integer btnId) throws JsonParseException, JsonMappingException, IOException {
		Map<String, Object> modelMap = new HashMap<>();
		if (btnId != null) {
			buttonInfoService.remove(btnId);
		}
		modelMap.put("status", "success");
		return modelMap;
	}

}
