package com.chinaitop.depot.system.controller;

import com.chinaitop.depot.system.model.DepotImage;
import com.chinaitop.depot.system.model.DepotImageExample;
import com.chinaitop.depot.system.service.DepotImageService;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.lang3.StringUtils;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RestController;

import javax.annotation.Resource;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

@RestController
@RequestMapping("/depotImage")
public class DepotImageController {

    @Resource
    private DepotImageService depotImageService;

    @RequestMapping(value="/list", method = RequestMethod.GET)
    @ApiOperation(value="粮库图片列表", notes = "查询粮库图片列表")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "orgId", value = "粮库编号", paramType = "query")
    })
    public List<DepotImage> list(Integer orgId) {
        DepotImageExample example = new DepotImageExample();
        if (orgId != null) {
            example.createCriteria().andOrgIdEqualTo(orgId);
        }
        return depotImageService.queryByExample(example);
    }

    @RequestMapping(value="/save", method = RequestMethod.POST)
    @ApiOperation(value="保存粮库图片", notes = "保存粮库图片")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "orgId", value = "粮库编号", paramType = "form"),
            @ApiImplicitParam(name = "imglistStr", value = "图片信息JSON串", paramType = "form")
    })
    public Map<String, Object> save(Integer orgId, String imglistStr) {
        Map<String, Object> modelMap = new HashMap<>();
        // JSON字符串转对象
        if (StringUtils.isNotBlank(imglistStr)) {
            String[] imgUrls = imglistStr.split(",");
            depotImageService.save(orgId, imgUrls);
        }
        modelMap.put("status", "success");
        return modelMap;
    }

}
