package com.chinaitop.depot.system.controller;

import com.chinaitop.depot.system.model.ButtonInfo;
import com.chinaitop.depot.system.model.ButtonInfoExample;
import com.chinaitop.depot.system.model.FuncInfo;
import com.chinaitop.depot.system.model.FuncInfoExample;
import com.chinaitop.depot.system.model.OrgInfo;
import com.chinaitop.depot.system.model.OrgInfoExample;
import com.chinaitop.depot.system.model.OrgInfoExample.Criteria;
import com.chinaitop.depot.system.model.OrgInfoVO;
import com.chinaitop.depot.system.model.RoleInfo;
import com.chinaitop.depot.system.service.ButtonInfoService;
import com.chinaitop.depot.system.service.FuncInfoService;
import com.chinaitop.depot.system.service.OrgInfoService;
import com.chinaitop.depot.system.service.RoleButtonService;
import com.chinaitop.depot.system.service.RoleFuncService;
import com.chinaitop.depot.system.service.RoleInfoService;
import com.chinaitop.depot.utils.BusinessUtils;
import com.chinaitop.depot.utils.DataSynchronization;
import com.chinaitop.depot.utils.FileUtil;
import com.fasterxml.jackson.core.JsonParseException;
import com.fasterxml.jackson.databind.JsonMappingException;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;

import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;

import org.apache.commons.lang.ObjectUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RestController;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;
import java.io.IOException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * 组织信息控制类
 * @author xueenxi
 *
 */
@RestController
@RequestMapping(value="/orgInfo")
@Api(description = "组织机构信息控制类")
public class OrgInfoController {
	
	@Resource
	private OrgInfoService orgInfoService;
	
	@Resource
	private RoleInfoService roleInfoService;
	
	@Resource
	private FuncInfoService funcInfoService;
	
	@Resource
	private ButtonInfoService buttonInfoService;
	
	@Resource
	private RoleFuncService roleFuncService;
	
	@Resource
	private RoleButtonService roleButtonService;

	@Value("${web.upload-path}")
	private String uploadPath;
	
	/**
	 * 查找组织信息
	 * @param pageNum 页码
	 * @param pageSize	每页条数
	 * @return
	 */
	@RequestMapping(value="/getOrg", method = RequestMethod.GET)
	@ApiOperation(value="查询组织机构信息", notes = "查询组织机构信息列表，支持分页")
	@ApiImplicitParams({
			@ApiImplicitParam(name = "pageNum", value = "页码", paramType = "query"),
			@ApiImplicitParam(name = "pageSize", value = "每页条数", paramType = "query"),
			@ApiImplicitParam(name = "orgName", value = "组织机构名", paramType = "query"),
			@ApiImplicitParam(name = "parentId", value = "上级组织机构编号", paramType = "query"),
			@ApiImplicitParam(name = "orgClassId", value = "单位类型(5317:单位,5318:库区)", paramType = "query")
	})
	public PageInfo<OrgInfo> getOrgInfo(Integer pageNum, Integer pageSize, Integer parentId, String orgName, Integer orgClassId) {
		OrgInfoExample example = new OrgInfoExample();
		Criteria criteria = example.createCriteria();
		if (parentId != null) {
			criteria.andParentIdEqualTo(parentId);
		}
		if (StringUtils.isNotBlank(orgName)) {
			criteria.andOrgNameLike("%" + orgName + "%");
		}
		if (orgClassId != null) {
			criteria.andOrgClassIdEqualTo(orgClassId);
		}
		if (pageNum!=null && pageSize!=null) {
			PageHelper.startPage(pageNum, pageSize);
		}
		List<OrgInfo> list = orgInfoService.queryByExample(example);
		PageInfo<OrgInfo> pageInfo = new PageInfo<OrgInfo>(list);
		return pageInfo;
	}

	@RequestMapping(value="/getTree", method = RequestMethod.GET)
	@ApiOperation(value="获取组织树", notes = "")
	public List<OrgInfo> getTree(String orgName) {
		OrgInfoExample example = new OrgInfoExample();
		Criteria criteria = example.createCriteria();
		if (StringUtils.isNotBlank(orgName)) {
			criteria.andOrgNameLike("%" + orgName + "%");
		}
		return orgInfoService.queryTreeByExample(example);
	}

	@RequestMapping(value="/edit", method = RequestMethod.GET)
	@ApiOperation(value="根据编号查找组织信息", notes = "")
	public OrgInfo edit(Integer orgId) {
		OrgInfo orgInfo = new OrgInfo();
		if (orgId != null) {
			orgInfo = orgInfoService.findById(orgId);
		}
		return orgInfo;
	}

	@RequestMapping(value="/save", method = RequestMethod.POST)
	@ApiOperation(value="保存组织信息", notes = "")
	public Map<String, Object> save(String orgInfoJson) throws JsonParseException, JsonMappingException, IOException {
		Map<String, Object> modelMap = new HashMap<>();
		// JSON字符串转对象
		ObjectMapper mapper = new ObjectMapper();
		OrgInfo orgInfo = (OrgInfo)mapper.readValue(orgInfoJson, OrgInfo.class);
        if (orgInfo.getOrgId() == null) {
        	if(!ObjectUtils.toString(orgInfo.getBirdsEye()).isEmpty()) {
        		String path = uploadPath+orgInfo.getBirdsEye();
        		byte[] file2byte = FileUtil.file2byte(path);
        		String hexString = FileUtil.toHexString(file2byte);
        		orgInfo.setWjl(hexString);
        	}
        	/* 保存数据  */
			orgInfoService.add(orgInfo);

			/* 生成库区标准编码 */
			String zzdm = orgInfo.getCreditCode();
			if ("".equals(zzdm)) {
				zzdm = orgInfo.getCompanyOrgCode();
			}
//			orgInfo.setKqbm(BusinessUtils.setBZkqbm(zzdm, orgInfo.getOrgId()));
			orgInfo.setOrgCode(String.format("%03d", orgInfo.getOrgId()));
			orgInfoService.updateSelective(orgInfo);
			
			//默认给新建的机构创建一个系统管理员角色
			RoleInfo roleInfo = new RoleInfo();
			roleInfo.setOrgId(orgInfo.getOrgId());
			roleInfo.setParentId(0);
			roleInfo.setRoleName("系统管理员");
			roleInfoService.add(roleInfo);
			
			FuncInfoExample exampleFunc = new FuncInfoExample();
			List<FuncInfo> funcList = funcInfoService.queryByExample(exampleFunc);
			roleFuncService.saveForList(roleInfo.getRoleId(), funcList);
			
			ButtonInfoExample exampleButton = new ButtonInfoExample();
			List<ButtonInfo> buttonList = buttonInfoService.queryByExample(exampleButton);
			roleButtonService.saveForList(roleInfo.getRoleId(), buttonList);
		} else {
			if(!ObjectUtils.toString(orgInfo.getBirdsEye()).isEmpty()) {
        		String path = uploadPath+orgInfo.getBirdsEye();
        		byte[] file2byte = FileUtil.file2byte(path);
        		String hexString = FileUtil.toHexString(file2byte);
        		orgInfo.setWjl(hexString);
        	}
//			orgInfo.setKqbm(BusinessUtils.setBZkqbm(orgInfo.getCreditCode(), orgInfo.getOrgId()));
//			orgInfoService.update(orgInfo);
		}
        if(orgInfo.getParentId() != 0) {//排除顶级机构
        	//查询父机构id以及层级
        	OrgInfo orgParentInfo = orgInfoService.findById(orgInfo.getParentId());
        	//查询同级机构已存在的级别后2位
        	Map<String,Object> map = new HashMap<String,Object>();
        	map.put("parentId", orgInfo.getParentId());
        	map.put("orgId", orgInfo.getOrgId());
        	List<OrgInfo> listMap = orgInfoService.getOrgLevel(map);
        	String level = "";
        	for(int i=1;i<100;i++) {
        		boolean flag = false;
        		if(i<10) {
        			level = "0"+i;
        		}else {
        			level = String.valueOf(i);
        		}
    			for(OrgInfo obj:listMap) {
    				if(obj != null) {
    					String levelCode = obj.getLevelCode();
    					if(!levelCode.contains(level)) {//不存在数据库里，说明可用
    						flag = true;
    						break;
    					}
    				}else {
    					flag = true;
						break;
    				}
    			}
        		if(flag) {
        			break;
        		}
        	}
        	if(5318 == orgInfo.getOrgClassId()) {//粮食企业的级别是在上级行政单位的级别上加2位顺序码
        		//更新本级机构层级
        		orgInfo.setLevelCode(orgParentInfo.getLevelCode()+level);
        		orgInfoService.update(orgInfo);
        		//orgInfoService.updateOrgLevel(orgParentInfo);
        	}else {
        		String parengtLevelCode = orgParentInfo.getLevelCode();
        		if(parengtLevelCode.equals(orgInfo.getLevelCode())) {//与父机构的级别一样时，在上级行政单位的级别上加2位顺序码
        			orgInfo.setLevelCode(orgParentInfo.getLevelCode()+level);
        			orgInfoService.update(orgInfo);
        		}
        	}
        }
		modelMap.put("status", "success");
		modelMap.put("newOrgId", orgInfo.getOrgId());
		return modelMap;
	}

	@RequestMapping(value="/remove", method = RequestMethod.POST)
	@ApiOperation(value="根据编号删除组织信息", notes = "")
	public Map<String, Object> remove(Integer orgId) {
		Map<String, Object> modelMap = new HashMap<>();
		if (orgId != null) {
			orgInfoService.remove(orgId);
		}
		modelMap.put("status", "success");
		return modelMap;
	}

	@RequestMapping(value="/updateSelective", method = RequestMethod.POST)
	@ApiOperation(value="根据编号修改组织信息", notes = "")
	public Map<String, Object> updateSelective(Integer orgId, String briefIntroduction) {
		Map<String, Object> modelMap = new HashMap<>();
		OrgInfo orgInfo = new OrgInfo();
		orgInfo.setOrgId(orgId);
		orgInfo.setBriefIntroduction(briefIntroduction);
		orgInfoService.updateSelective(orgInfo);
		modelMap.put("status", "success");
		return modelMap;
	}

	//所属组织
	@RequestMapping("/getTreeListByOrgId")
	@ApiOperation(value="初始化对应组织信息", notes = "")
	public Map<String, Object> getTreeListByOrgId(HttpServletRequest request) {
		Map<String, Object> modelMap = new HashMap<>();
		try {
			ObjectMapper mapper = new ObjectMapper();
			String orgInfoObject = ObjectUtils.toString(request.getSession().getAttribute("orgInfo"),"");
			OrgInfo depotInfo = mapper.readValue(orgInfoObject, OrgInfo.class);
			Integer orgId = depotInfo.getOrgId();
			
			if (orgId == null) {
				return modelMap;
			}
			//List<OrgInfo> list = orgInfoService.getTreeListByOrgId(orgId);
			List<OrgInfo> list = orgInfoService.getAllChildren(orgId,null);
			List<OrgInfoVO> voList = new ArrayList<OrgInfoVO>();
			for (OrgInfo orgInfo : list) {
				OrgInfoVO orgInfoVO = new OrgInfoVO();
				orgInfoVO.setParentId(orgInfo.getParentId());
				orgInfoVO.setId(orgInfo.getOrgId());
				orgInfoVO.setName(orgInfo.getOrgName());
				voList.add(orgInfoVO);
			}
			modelMap.put("orgList", voList);
			modelMap.put("orgId", orgId);
		} catch (IOException e) {
			e.printStackTrace();
		}
		return modelMap;
	}
	
	//库点信息同步
	@RequestMapping("/orgInfoSynchronization")
	@ApiOperation(value="同步组织信息", notes = "")
	public void orgInfoSynchronization() {
		SimpleDateFormat format = new SimpleDateFormat("yyyy-MM-dd'T'HH:mm:ss.SSSZ");
		List list = DataSynchronization.queryDepotInfo();
		List<OrgInfo> orglist = orgInfoService.getAllDepotInfo();
		for(OrgInfo orgInfo:orglist) {
			int num = 0;
			for(int i=0;i<list.size();i++) {
				Map<String,Object> map = (Map<String,Object>)list.get(i);
				String kdbm = ObjectUtils.toString(map.get("kdbm"),"");//库点编码
				String refreshTime = ObjectUtils.toString(map.get("refreshTime"),"");
				if(kdbm.equals(ObjectUtils.toString(orgInfo.getOrgId(),""))) {
					if(!refreshTime.equals(format.format(orgInfo.getUpdateTime()))) {
						DataSynchronization.orgInfoSynchronizationUpdate(map, orgInfo);
					}
					break;
				}else if(num == (list.size()-1)){
					DataSynchronization.orgInfoSynchronizationInsert(orgInfo);
				}else {
					num = num+1;
				}
			}
		}
	}

	@RequestMapping(value="/updateWjl", method = RequestMethod.GET)
	@ApiOperation(value="把没有生成文件流的数据生成文件流", notes = "")
	public Map<String, Object> updateWjl() {
		Map<String, Object> modelMap = new HashMap<>();
		OrgInfoExample example = new OrgInfoExample();
		List<OrgInfo> list = orgInfoService.queryByExample(example);
		for(OrgInfo orgInfo:list) {
			if(!ObjectUtils.toString(orgInfo.getBirdsEye()).isEmpty()) {
				try {
					String path = uploadPath+orgInfo.getBirdsEye();
					byte[] file2byte = FileUtil.file2byte(path);
					String hexString = FileUtil.toHexString(file2byte);
					orgInfo.setWjl(hexString);
					orgInfoService.update(orgInfo);
				} catch (Exception e) {
					e.printStackTrace();
				}
        	}
		}
		modelMap.put("status", "success");
		return modelMap;
	}
	
	 //角色
   /* @RequestMapping("/getTreeAreaList")
    public Map<String, Object> getTreeAreaList(HttpServletRequest request) {
    	Map<String, Object> modelMap = new HashMap<>();
    	List<RoleInfo> lists = orgInfoService.getAllChildren(2905,null);
    	List<BasicEnumVo> voList = new ArrayList<BasicEnumVo>();
    	for (RoleInfo roleInfo : lists) {
    		BasicEnumVo basicEnumVo = new BasicEnumVo();
    		basicEnumVo.setId(basicEnum.getEnumid());
    		basicEnumVo.setName(basicEnum.getEnumname());
    		basicEnumVo.setParentid(basicEnum.getParentid());
    		voList.add(basicEnumVo);
    	}
    	modelMap.put("enumListArea", voList);
    	modelMap.put("enumId", 2905);
    	return modelMap;
    }*/
}
