package com.chinaitop.depot.system.controller;

import com.chinaitop.depot.system.model.RoleButton;
import com.chinaitop.depot.system.model.RoleButtonExample;
import com.chinaitop.depot.system.model.RoleButtonExample.Criteria;
import com.chinaitop.depot.system.model.UserInfo;
import com.chinaitop.depot.system.service.RoleButtonService;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;

import org.apache.commons.lang.ObjectUtils;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;

import java.io.IOException;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * 角色按钮权限控制类
 * @author xueenxi
 *
 */
@RestController
@RequestMapping(value="/roleButton")
public class RoleButtonController {
	
	@Resource
	private RoleButtonService roleButtonService;
	
	/**
	 * 获取角色按钮权限列表
	 * @param pageNum	页码
	 * @param pageSize	每页条数
	 * @param roleId	角色编号
	 * @return
	 */
	@RequestMapping(value="/getRoleButton")
	public PageInfo<RoleButton> getRoleButton(Integer pageNum, Integer pageSize, Integer roleId) {
		RoleButtonExample example = new RoleButtonExample();
		Criteria criteria = example.createCriteria();
		if (roleId != null) {
			criteria.andRoleIdEqualTo(roleId);
		}
		if (pageNum!=null && pageSize!=null) {
			PageHelper.startPage(pageNum, pageSize);
		}
		List<RoleButton> list = roleButtonService.queryByExample(example);
		PageInfo<RoleButton> pageInfo = new PageInfo<RoleButton>(list);
		return pageInfo;
	}
	
	/**
	 * 查询当前用户所有操作权限
	 * @param request
	 * @return
	 */
	@RequestMapping(value="/getByUser")
	public List<RoleButton> getByUser(HttpServletRequest request) {
		try {
			ObjectMapper mapper = new ObjectMapper();
			String userInfoObject = ObjectUtils.toString(request.getSession().getAttribute("userInfo"),"");
			if(!userInfoObject.isEmpty()) {
				UserInfo userInfo = mapper.readValue(userInfoObject, UserInfo.class);
				if (userInfo != null) {
					return roleButtonService.queryByUser(userInfo);
				}
			}
		} catch (IOException e) {
			e.printStackTrace();
		}
		return null;
	}
	
	/**
	 * 保存角色按钮权限
	 * @param roleId	角色编号
	 * @param btnIds	按钮编号，多个之间用逗号隔开
	 * @return
	 */
	@RequestMapping(value="/save")
	public Map<String, Object> save(Integer roleId, String btnIds) {
		String[] btnIdList = btnIds.split(",");
		roleButtonService.save(roleId, btnIdList);
		return new HashMap<String, Object>();
	}

}
