package com.chinaitop.depot.system.controller;

import com.chinaitop.depot.system.model.FuncInfo;
import com.chinaitop.depot.system.model.RoleFunc;
import com.chinaitop.depot.system.model.UserInfo;
import com.chinaitop.depot.system.service.RoleFuncService;
import com.fasterxml.jackson.databind.ObjectMapper;

import org.apache.commons.lang.ObjectUtils;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;

import java.io.IOException;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * 角色功能权限控制类
 * @author xueenxi
 *
 */
@RestController
@RequestMapping(value="/roleFunc")
public class RoleFuncController {
	
	@Resource
	private RoleFuncService roleFuncService;
	
	/**
	 * 根据角色编号获取列表数据
	 * @param roleId	角色编号
	 * @return List<RoleFunc>	角色功能列表
	 */
	@RequestMapping(value="/getByRoleId")
	public List<RoleFunc> getByRoleId(Integer roleId) {
		// TODO Auto-generated method stub
		return roleFuncService.queryByRoleId(roleId);
	}
	
	/**
	 * 获取当前用户所有功能权限
	 * @param request
	 * @return	List<FuncInfo>	功能列表
	 */
	@RequestMapping(value="/getFuncByUserId")
	public List<FuncInfo> getFuncByUserId(HttpServletRequest request) {
		try {
			ObjectMapper mapper = new ObjectMapper();
			String userInfoObject = ObjectUtils.toString(request.getSession().getAttribute("userInfo"),"");
			if(!userInfoObject.isEmpty()) {
				UserInfo userInfo = mapper.readValue(userInfoObject, UserInfo.class);
				if (userInfo != null) {
					return roleFuncService.queryFuncByUserId(userInfo.getUserId());
				}
			}
		} catch (IOException e) {
			e.printStackTrace();
		}
		return null;
	}

	/**
	 * 保存角色功能权限
	 * @param roleId	角色编号
	 * @param funcIds	功能编号，多个之间用逗号隔开
	 * @return
	 */
	@RequestMapping(value="/save")
	public Map<String, Object> save(Integer roleId, String funcIds) {
		String[] funcIdList = funcIds.split(",");
		roleFuncService.save(roleId, funcIdList);
		return new HashMap<String, Object>();
	}

}
