package com.chinaitop.depot.system.controller;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.chinaitop.depot.system.model.OrgInfo;
import com.chinaitop.depot.system.model.RoleInfo;
import com.chinaitop.depot.system.model.RoleInfoExample;
import com.chinaitop.depot.system.model.RoleInfoExample.Criteria;
import com.chinaitop.depot.system.model.RoleInfoVo;
import com.chinaitop.depot.system.service.RoleButtonService;
import com.chinaitop.depot.system.service.RoleFuncService;
import com.chinaitop.depot.system.service.RoleInfoService;
import com.chinaitop.depot.utils.NetStateUtil;
import com.chinaitop.depot.webservice.ExchangeServer;
import com.chinaitop.depot.webservice.IExchangeService;
import com.fasterxml.jackson.core.JsonParseException;
import com.fasterxml.jackson.databind.JsonMappingException;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;

import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;

import org.apache.commons.lang.ObjectUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RestController;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;
import java.io.IOException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

@RestController
@RequestMapping(value="/roleInfo")
public class RoleInfoController {
	
	@Resource
	private RoleInfoService roleInfoService;
	
	@Resource
	private RoleFuncService roleFuncService;
	
	@Resource
	private RoleButtonService roleButtonService;
	
	@Value("${FLATID}")
	private String FLATID;
	
	@Value("${KEY}")
	private String KEY;
	
	@Value("${wsdlUrl}")
	private String wsdlUrl;
	
	/**
	 * 查找角色信息
	 * @param pageNum	页码
	 * @param pageSize	每页条数
	 * @return
	 */
	@RequestMapping(value="/getRole", method=RequestMethod.GET)
	@ApiOperation(value="角色信息列表", notes = "支持分页")
	public PageInfo<RoleInfo> getRoleInfo(HttpServletRequest request, Integer pageNum, Integer pageSize, Integer parentId, String roleName) {
		RoleInfoExample example = new RoleInfoExample();
		Criteria criteria = example.createCriteria();
		if (parentId != null) {
			criteria.andParentIdEqualTo(parentId);
		}
		if (StringUtils.isNotBlank(roleName)) {
			criteria.andRoleNameLike("%" + roleName + "%");
		}
		if (pageNum!=null && pageSize!=null) {
			PageHelper.startPage(pageNum, pageSize);
		}
		List<RoleInfo> list = roleInfoService.queryByExample(example);	
		
		ObjectMapper mapper = new ObjectMapper();
		String orgInfoObject = ObjectUtils.toString(request.getSession().getAttribute("orgInfo"),"");
		if(!orgInfoObject.isEmpty()) {
			try {
				OrgInfo orgInfo = mapper.readValue(orgInfoObject, OrgInfo.class);
				parentId = orgInfo.getParentId();
			} catch (Exception e) {
				e.printStackTrace();
			}
		}
		
		PageInfo<RoleInfo> pageInfo = null;
		try {
			if(parentId == 0) {//是顶级机构，则获取出入库的角色信息
				if("正常".equals(NetStateUtil.connectingAddress(wsdlUrl))) {
					ExchangeServer wb = new ExchangeServer();
					IExchangeService action = wb.getExchangeServiceImplPort();
					//获取系统所有角色
					String roles = action.getAllRoles(FLATID, KEY);
					JSONObject obj = JSONObject.parseObject(roles);  
					String result = obj.getString("result");
					JSONArray data = JSONArray.parseArray(result);
					for(int i=0;i<data.size();i++) {
						RoleInfo roleInfo = new RoleInfo();
						Map<String,Object> map = (Map<String,Object>)data.get(i);
						roleInfo.setRoleName(ObjectUtils.toString(map.get("roleName"),""));
						roleInfo.setRemark(ObjectUtils.toString(map.get("roleId"),""));
						roleInfo.setParentId(0);
						roleInfo.setRoleId(100+i);
						list.add(roleInfo);
					}
				}
			}
		} finally {
			pageInfo = new PageInfo<RoleInfo>(list);
		}
		
		return pageInfo;
	}

	/**
	 * 根据编号查找角色信息
	 * @param roleId	角色编号
	 * @return	RoleInfo 角色信息
	 */
	@RequestMapping(value="/edit", method=RequestMethod.GET)
	@ApiOperation(value="查询一条角色信息", notes = "支持分页")
	public RoleInfo edit(Integer roleId) {
		RoleInfo roleInfo = new RoleInfo();
		if (roleId != null) {
			roleInfo = roleInfoService.findById(roleId);
		}
		return roleInfo;
	}

	/**
	 * 验证角色名称是否可用
	 * @param roleName	角色名称
	 * @param roleId	角色id
	 * @param status	类型：1 粮库、2 出入库
	 * @return
	 */
	@RequestMapping(value="/validRoleName", method = RequestMethod.GET)
	@ApiOperation(value="验证角色名称", notes = "验证角色名称是否可用")
	@ApiImplicitParams({
			@ApiImplicitParam(name = "roleName", value = "移动端类型", paramType = "query"),
			@ApiImplicitParam(name = "roleId", value = "角色id", paramType = "query"),
			@ApiImplicitParam(name = "status", value = "类型：1 粮库、2 出入库", paramType = "query")
	})
	public Boolean validRoleName(String roleName,String roleId, String status) {
		if("1".equals(status)) {
			return roleInfoService.validRoleName(roleName,roleId);
		}else {
			try {
				if("正常".equals(NetStateUtil.connectingAddress(wsdlUrl))) {
					ExchangeServer wb = new ExchangeServer();
					IExchangeService action = wb.getExchangeServiceImplPort();
					//获取系统所有角色
					String roles = action.getAllRoles(FLATID, KEY);
					JSONObject obj = JSONObject.parseObject(roles);  
					String result = obj.getString("result");
					JSONArray data = JSONArray.parseArray(result);
					for(int i=0;i<data.size();i++) {
						Map<String,Object> map = (Map<String,Object>)data.get(i);
						if(ObjectUtils.toString(map.get("roleName"),"").equals(roleName) && !ObjectUtils.toString(map.get("roleId"),"").equals(roleId)) {
							return false;
						}
					}
				}
			}catch (Exception e) {
				e.printStackTrace();
			}
			return true;
		}
	}

	/**
	 * 保存角色信息
	 * @param roleInfoJson	角色信息json字符串
	 * @return
	 * @throws JsonParseException
	 * @throws JsonMappingException
	 * @throws IOException
	 */
	@RequestMapping(value="/save", method = RequestMethod.POST)
	@ApiOperation(value="保存角色名称", notes = "")
	public Map<String, Object> save(HttpServletRequest request, String roleInfoJson)
			throws JsonParseException, JsonMappingException, IOException {
		Map<String, Object> modelMap = new HashMap<>();
		// JSON字符串转对象
		ObjectMapper mapper = new ObjectMapper();
		RoleInfo roleInfo = (RoleInfo)mapper.readValue(roleInfoJson, RoleInfo.class);
		Integer orgId = (Integer)request.getSession().getAttribute("orgId");
		roleInfo.setOrgId(orgId);
		if (roleInfo.getRoleId() == null) {
			roleInfoService.add(roleInfo);
		} else {
			roleInfoService.update(roleInfo);
		}
		modelMap.put("status", "success");
		return modelMap;
	}

	@RequestMapping(value="/saveCrk", method = RequestMethod.POST)
	@ApiOperation(value="保存出入库角色信息", notes = "")
	public Map<String, Object> saveCrk(HttpServletRequest request, String roleId, String roleName) {
		Map<String, Object> modelMap = new HashMap<>();
		if("正常".equals(NetStateUtil.connectingAddress(wsdlUrl))) {
			ExchangeServer wb = new ExchangeServer();
			IExchangeService action = wb.getExchangeServiceImplPort();
			if(roleId.isEmpty()) {//新增
				String json = "{roleId:\"\",roleName:\""+roleName+"\",notes:\"\"}";
				String result = action.addRole(FLATID, KEY, json);
				JSONObject obj = JSONObject.parseObject(result);  
				String status = obj.getString("success");//0失败 1成功
				modelMap.put("status", status);
			} else {//修改
				String json = "{roleId:\""+roleId+"\",roleName:\""+roleName+"\",notes:\"\"}";
				String result = action.updateRole(FLATID, KEY, json);
				JSONObject obj = JSONObject.parseObject(result);  
				String status = obj.getString("success");//0失败 1成功
				modelMap.put("status", status);
			}
		}else {
			modelMap.put("status", 0);
		}
		
		return modelMap;
	}

	@RequestMapping(value="/remove", method = RequestMethod.POST)
	@ApiOperation(value="根据编号删除角色", notes = "")
	public Map<String, Object> remove(Integer roleId) {
		Map<String, Object> modelMap = new HashMap<>();
		if (roleId != null) {
			roleInfoService.remove(roleId);
			roleFuncService.remove(roleId);
			roleButtonService.remove(roleId);
		}
		modelMap.put("status", "success");
		return modelMap;
	}

	@RequestMapping(value="/removeCrk", method = RequestMethod.POST)
	@ApiOperation(value="根据编号删除出入库角色", notes = "")
	public Map<String, Object> removeCrk(String roleId) {
		Map<String, Object> modelMap = new HashMap<>();
		if (roleId != null) {
			ExchangeServer wb = new ExchangeServer();
			IExchangeService action = wb.getExchangeServiceImplPort();
			String result = action.deleteRole(FLATID, KEY, roleId);
			JSONObject obj = JSONObject.parseObject(result);  
			String status = obj.getString("success");//0失败 1成功
			modelMap.put("status", status.equals("1") ? "success":"fail");
		}
		return modelMap;
	}
	
	@RequestMapping(value="/getRoleTree", method = RequestMethod.GET)
	@ApiOperation(value="初始化角色", notes = "")
    public Map<String, Object> getRoleTree(HttpServletRequest request) {
		RoleInfoExample example = new RoleInfoExample();
		List<RoleInfo> list = roleInfoService.queryByExample(example);	
		Map<String, Object> modelMap = new HashMap<>();
		List<RoleInfoVo> voList = new ArrayList<RoleInfoVo>();
		Map<Integer, String> roleNameObj = new HashMap<Integer, String>();
		for (RoleInfo roleInfo : list) {
			RoleInfoVo roleInfoVo = new RoleInfoVo();
			roleInfoVo.setId(roleInfo.getRoleId());
			roleInfoVo.setName(roleInfo.getRoleName());
			roleInfoVo.setParentId(roleInfo.getParentId());
			voList.add(roleInfoVo);
			roleNameObj.put(roleInfo.getRoleId(), roleInfo.getRoleName());
		}
		modelMap.put("roleList", voList);
		modelMap.put("roleNameObj", roleNameObj);
		modelMap.put("roleId", 0);
    	return modelMap;
    }
	
	@RequestMapping("/getRoleTreeByOrg")
	public Map<String, Object> getRoleTreeByOrg(Integer orgId) {
		Map<String, Object> map = new HashMap<>();
		map.put("orgId", orgId);
		List<RoleInfo> list = roleInfoService.getRoleTreeByOrg(map);	
		Map<String, Object> modelMap = new HashMap<>();
		List<RoleInfoVo> voList = new ArrayList<RoleInfoVo>();
		Map<Integer, String> roleNameObj = new HashMap<Integer, String>();
		for (RoleInfo roleInfo : list) {
			RoleInfoVo roleInfoVo = new RoleInfoVo();
			roleInfoVo.setId(roleInfo.getRoleId());
			roleInfoVo.setName(roleInfo.getRoleName());
			roleInfoVo.setParentId(roleInfo.getParentId());
			voList.add(roleInfoVo);
			roleNameObj.put(roleInfo.getRoleId(), roleInfo.getRoleName());
		}
		
		//追加出入库的角色信息
		if("正常".equals(NetStateUtil.connectingAddress(wsdlUrl))) {
			try {
				ExchangeServer wb = new ExchangeServer();
				IExchangeService action = wb.getExchangeServiceImplPort();
				//获取系统所有角色
				String roles = action.getAllRoles(FLATID, KEY);
				JSONObject obj = JSONObject.parseObject(roles);  
				String result = obj.getString("result");
				JSONArray data = JSONArray.parseArray(result);
				for(int i=0;i<data.size();i++) {
					RoleInfoVo roleInfoVo = new RoleInfoVo();
					Map<String,Object> maps = (Map<String,Object>)data.get(i);
					roleInfoVo.setName(ObjectUtils.toString(maps.get("roleName"),""));
					roleInfoVo.setCrkId(ObjectUtils.toString(maps.get("roleId"),""));
					roleInfoVo.setParentId(0);
					voList.add(roleInfoVo);
				}
			} catch (Exception e) {
				e.printStackTrace();
			}
		}
		
		modelMap.put("roleList", voList);
		modelMap.put("roleNameObj", roleNameObj);
		modelMap.put("roleId", 0);
		return modelMap;
	}

	@RequestMapping("/getCrkFunc")
	public Map<String, Object> getCrkFunc() {
		Map<String, Object> modelMap = new HashMap<>();
		List<Map<String, Object>> funcList = new ArrayList<Map<String, Object>>();
		List<Map<String, Object>> btnList = new ArrayList<Map<String, Object>>();
		//追加出入库的角色信息
		if("正常".equals(NetStateUtil.connectingAddress(wsdlUrl))) {
			try {
				ExchangeServer wb = new ExchangeServer();
				IExchangeService action = wb.getExchangeServiceImplPort();
				//获取所有功能信息
				String funcs = action.getAllPowers(FLATID, KEY);
				JSONObject obj = JSONObject.parseObject(funcs);  
				String result = obj.getString("result");
				JSONArray data = JSONArray.parseArray(result);
				for(int i=0;i<data.size();i++) {
					Map<String,Object> map = (Map<String,Object>)data.get(i);
					if(ObjectUtils.toString(map.get("isMenu"),"").equals("0")) {//isMenu=1时表示菜单；isMenu=0时表示按钮级权限
						Map<String, Object> btnMap = new HashMap<String, Object>();
						btnMap.put("btnName", ObjectUtils.toString(map.get("funcName"),""));
						btnMap.put("btnId", ObjectUtils.toString(map.get("funcId"),""));
						btnMap.put("funcId", ObjectUtils.toString(map.get("parentId"),"")==""?"0":map.get("parentId"));
						btnList.add(btnMap);
					}else if(ObjectUtils.toString(map.get("isMenu"),"").equals("1")){
						Map<String, Object> funcMap = new HashMap<String, Object>();
						funcMap.put("funcName", ObjectUtils.toString(map.get("funcName"),""));
						funcMap.put("funcId", ObjectUtils.toString(map.get("funcId"),""));
						funcMap.put("parentId", ObjectUtils.toString(map.get("parentId"),"")==""?"0":map.get("parentId"));
						funcList.add(funcMap);
					}
				}
			} catch (Exception e) {
				e.printStackTrace();
			}
		}
		modelMap.put("funcList", funcList);
		modelMap.put("btnList", btnList);
		return modelMap;
	}
	
	@RequestMapping(value="/getByRoleIdCrk")
	public List<Map<String, Object>> getByRoleIdCrk(String roleId) {
		List<Map<String, Object>> funcList = new ArrayList<Map<String, Object>>();
		ExchangeServer wb = new ExchangeServer();
		IExchangeService action = wb.getExchangeServiceImplPort();
		String roleAllPowers = action.getRoleAllPowers(FLATID, KEY, roleId);
		JSONObject obj = JSONObject.parseObject(roleAllPowers);  
		String result = obj.getString("result");
		JSONArray data = JSONArray.parseArray(result);
		for(int i=0;i<data.size();i++) {
			Map<String,Object> map = (Map<String,Object>)data.get(i);
			Map<String, Object> funcMap = new HashMap<String, Object>();
			funcMap.put("funcId", ObjectUtils.toString(map.get("funcId"),""));
			funcMap.put("roleId", ObjectUtils.toString(map.get("roleId"),""));
			funcList.add(funcMap);
		}
		return funcList;
	}
	
	@RequestMapping(value="/saveRoleButtonCrk")
	public Map<String, Object> saveRoleButtonCrk(String roleId, String btnIds) {
		Map<String, Object> modelMap = new HashMap<>();
		String[] btnIdList = btnIds.split(",");
		String str = "[";
		for(int i=0;i<btnIdList.length;i++) {
			if(i<btnIdList.length-1) {
				str=str+"\""+btnIdList[i]+"\",";
			}else {
				str=str+"\""+btnIdList[i]+"\"]";
			}
		}
		if(str == "[") {
			str = "[]";
		}
		String json = "{roleId:\""+roleId+"\", funcIds:[], btnIds:"+str+"}";
		ExchangeServer wb = new ExchangeServer();
		IExchangeService action = wb.getExchangeServiceImplPort();
		String result = action.saveRolePowers(FLATID, KEY, json);
		JSONObject obj = JSONObject.parseObject(result);  
		String status = obj.getString("success");//0失败 1成功
		modelMap.put("status", status.equals("1") ? "success":"fail");
		return modelMap;
	}

}
