package com.chinaitop.depot.system.controller;

import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.OutputStream;
import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.UUID;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.commons.lang.ObjectUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.ResponseBody;
import org.springframework.web.bind.annotation.RestController;

import com.chinaitop.depot.system.model.TUrlConfig;
import com.chinaitop.depot.system.service.TUrlConfigService;
import com.chinaitop.depot.utils.FileExport;
import com.chinaitop.depot.utils.NetStateUtil;
import com.fasterxml.jackson.core.JsonParseException;
import com.fasterxml.jackson.databind.JsonMappingException;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;

import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;

/**
 * url配置控制类
 * @author hf
 *
 */
@RestController
@RequestMapping(value="/urlConfig")
@Api(description = "url配置控制类")
public class UrlConfigController {
	
	@Autowired
	private TUrlConfigService tUrlConfigService;
	
	@Value("${web.upload-path}")
	private String uploadPath;

	/**
	 * 查找url配置信息
	 * @param pageNum	页码
	 * @param pageSize	每页条数
	 * @return
	 */
	@RequestMapping(value="/getUrlConfig", method = RequestMethod.GET)
	@ApiOperation(value="查找url配置信息", notes = "查找url配置信息列表，支持分页")
	@ApiImplicitParams({
			@ApiImplicitParam(name = "pageNum", value = "页码", paramType = "query"),
			@ApiImplicitParam(name = "pageSize", value = "每页条数", paramType = "query"),
			@ApiImplicitParam(name = "intranetUrl", value = "内网url", paramType = "query"),
			@ApiImplicitParam(name = "orgName", value = "所属库", paramType = "query"),
			@ApiImplicitParam(name = "updateStatus", value = "更新状态", paramType = "query")
	})
	public PageInfo<TUrlConfig> getUrlConfig(HttpServletRequest request, Integer pageNum, Integer pageSize,
    	String intranetUrl, String orgName, String updateStatus) {
		
		if (pageNum!=null && pageSize!=null) {
			PageHelper.startPage(pageNum, pageSize);
		}
		
		HashMap<String, Object> map = new HashMap<String, Object>();
		map.put("intranetUrl", intranetUrl);
		map.put("orgName", orgName);
		map.put("updateStatus", updateStatus);
		List<TUrlConfig> listAll=tUrlConfigService.queryAll(map);
		PageInfo<TUrlConfig> pageInfo = new PageInfo<TUrlConfig>(listAll);
		return pageInfo;
	}

	/**
	 * 验证url是否可用
	 * @param intranetUrl	内网url
	 * @param id	主键id
	 * @return
	 */
	@RequestMapping(value="/validIntranetUrl", method = RequestMethod.GET)
	@ApiOperation(value="验证url", notes = "验证url是否可用")
	@ApiImplicitParams({
			@ApiImplicitParam(name = "intranetUrl", value = "内网url", paramType = "query"),
			@ApiImplicitParam(name = "id", value = "主键id", paramType = "query")
	})
	public Boolean validIntranetUrl(String intranetUrl,String id) {
		return tUrlConfigService.validIntranetUrl(intranetUrl,id);
	}
	
	/**
	 * 根据主键id查找url配置信息
	 * @param id 主键id
	 * @return	UserInfo	用户信息
	 */
	@RequestMapping(value="/edit", method = RequestMethod.GET)
	@ApiOperation(value="编辑url配置信息", notes = "编辑url配置信息，用于新增或者修改")
	@ApiImplicitParams({
			@ApiImplicitParam(name = "id", value = "主键id", paramType = "query")
	})
	public TUrlConfig edit(String id) {
		TUrlConfig tUrlConfig = new TUrlConfig();
		if (id != null && !id.isEmpty()) {
			tUrlConfig = tUrlConfigService.findById(id);
		}
		return tUrlConfig;
	}

	/**
	 * 保存url配置信息
	 * @param urlConfigJson	url配置信息json字符串
	 * @return
	 * @throws JsonParseException
	 * @throws JsonMappingException
	 * @throws IOException
	 */
	@RequestMapping(value="/save", method = RequestMethod.POST)
	@ApiOperation(value="保存url配置信息", notes = "保存url配置信息，新增或者修改")
	@ApiImplicitParams({
			@ApiImplicitParam(name = "urlConfigJson", value = "url配置信息JSON串", paramType = "form")
	})
	public Map<String, Object> save(String urlConfigJson) throws JsonParseException, JsonMappingException, IOException {
		Map<String, Object> modelMap = new HashMap<>();
		// JSON字符串转对象
		ObjectMapper mapper = new ObjectMapper();
		TUrlConfig tUrlConfig = (TUrlConfig)mapper.readValue(urlConfigJson, TUrlConfig.class);
		if(tUrlConfig.getId() == null){
			String id = UUID.randomUUID().toString().replace("-", "");//动态生成32随机字符串
			tUrlConfig.setId(id);
			tUrlConfig.setTime(new Date());
			tUrlConfigService.add(tUrlConfig);
		}else{
			tUrlConfigService.update(tUrlConfig);
		}
		modelMap.put("status", "success");
		return modelMap;
	}
	
	/**
	 * 根据主键删除url配置
	 * @param id	主键
	 * @return
	 */
	@RequestMapping(value="/remove", method = RequestMethod.POST)
	@ApiOperation(value="删除url配置", notes = "根据主键删除url配置")
	@ApiImplicitParams({
			@ApiImplicitParam(name = "id", value = "主键id", paramType = "form")
	})
	public Map<String, Object> remove(String id) {
		Map<String, Object> modelMap = new HashMap<>();
		if (id != null && !id.isEmpty()) {
			tUrlConfigService.remove(id);
		}
		modelMap.put("status", "success");
		return modelMap;
	}
	
	/**
	 * 导出数据
	 * @return
	 */
	@RequestMapping(value="/exportTxt", method = RequestMethod.POST, produces ="application/json;charset=UTF-8")
	@ApiOperation(value="导出数据到TXT", notes = "")
	@ResponseBody
	public void exportTxt(HttpServletRequest request, HttpServletResponse response) {
		response.setContentType("charset=utf-8");
		response.setContentType("bin");
		response.setContentType("application/force-download");// 设置强制下载不打开  	      
		
		String path = uploadPath+"/kettle.txt";
		File filenameShip = new File(path);
		FileExport.createFile(path);
		Map<String, Object> map = new HashMap<String, Object>();
		map.put("userType", "1");
		List<Map<String, Object>> list = new ArrayList<Map<String, Object>>();
		List<Map<String, Object>> dataList = tUrlConfigService.queryExportInfo(map);
		for(Map<String, Object> m : dataList) {
			HashMap<String, Object> map2 = new HashMap<String, Object>();
			map2.put("depot_name", m.get("depotname"));
			map2.put("depot_id", m.get("depotid"));
			map2.put("depot_server_ip", NetStateUtil.getIpFromUrl(ObjectUtils.toString(m.get("depoturl"))));
			map2.put("depot_start_day", "");
			map2.put("depot_end_day", "");
			map2.put("back_time", 30);
			list.add(map2);
		}
		
		String outputFileName = "kettle.txt"; //输出文件的文件名
		
		try {
			FileExport.writeTxtFileFor(list, filenameShip);//写入文件
			
			//设置火狐编码
			String agent = request.getHeader("User-Agent");
			if (null != agent && "" != agent) {
			    agent = agent.toLowerCase();
			    if (agent.indexOf("firefox") != -1) {
			    	outputFileName = new String(outputFileName.getBytes("UTF-8"),"ISO-8859-1");  
			    } else {
			    	outputFileName = java.net.URLEncoder.encode(outputFileName,"UTF-8");  
			    }
			}
			//读取要下载的文件，保存到文件输入流
			FileInputStream in = new FileInputStream(path);
	        response.setHeader("Content-Disposition", "attachment; filename=" + outputFileName);
	        //创建输出流
	        OutputStream out = response.getOutputStream();
	        //创建缓冲区
	        byte buffer[] = new byte[1024];
	        int len = 0;
	        //循环将输入流中的内容读取到缓冲区当中
	        while((len=in.read(buffer))>0){
	            //输出缓冲区的内容到浏览器，实现文件下载
	            out.write(buffer, 0, len);
	        }
	        out.close();
	        //关闭文件输入流
	        in.close();

		} catch (IOException e) {
			e.printStackTrace();
		}
		
	}

	@RequestMapping(value="/getUrl", method = RequestMethod.GET)
	@ApiOperation(value="查询一条配置的应急端系统URL", notes = "只返回内网URL配置")
	@ApiImplicitParams({
		@ApiImplicitParam(name = "orgId", value = "粮库id", paramType = "query"),
		@ApiImplicitParam(name = "userType", value = "类型(1:库级,2:出入库)", paramType = "query")
	})
	public String getUrl(String orgId, String userType) {
		String url = "";
		try {
			url = tUrlConfigService.getUrl(orgId, userType);
		} catch (Exception e) {
			e.printStackTrace();
		}
		return url;
	}
}
