package com.chinaitop.depot.system.controller;

import java.awt.image.BufferedImage;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.time.Duration;
import java.time.Instant;
import java.time.LocalDateTime;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Timer;
import java.util.TimerTask;

import javax.annotation.Resource;
import javax.imageio.ImageIO;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpSession;

import org.apache.commons.codec.binary.Base64;
import org.apache.commons.lang.ObjectUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.http.MediaType;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.chinaitop.depot.system.model.OrgInfo;
import com.chinaitop.depot.system.model.TUrlConfig;
import com.chinaitop.depot.system.model.UserInfo;
import com.chinaitop.depot.system.model.UserInfoExample;
import com.chinaitop.depot.system.model.UserInfoExample.Criteria;
import com.chinaitop.depot.system.service.OrgInfoService;
import com.chinaitop.depot.system.service.RoleInfoService;
import com.chinaitop.depot.system.service.TUrlConfigService;
import com.chinaitop.depot.system.service.UserInfoService;
import com.chinaitop.depot.system.service.UserOrgService;
import com.chinaitop.depot.system.service.UserRoleService;
import com.chinaitop.depot.utils.MySessionContext;
import com.chinaitop.depot.utils.NetStateUtil;
import com.chinaitop.depot.utils.ParameterUtil;
import com.chinaitop.depot.utils.RedisUtil;
import com.chinaitop.depot.utils.SHA1Util;
import com.chinaitop.depot.utils.SecurityCode;
import com.chinaitop.depot.utils.SecurityCode.SecurityCodeLevel;
import com.chinaitop.depot.utils.SecurityImage;
import com.chinaitop.depot.webservice.ExchangeServer;
import com.chinaitop.depot.webservice.IExchangeService;

import com.fasterxml.jackson.core.JsonParseException;
import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.databind.JsonMappingException;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;

import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;

/**
 * 用户信息控制类
 * @author xueenxi
 *
 */
@RestController
@RequestMapping(value="/userInfo")
@Api(description = "用户信息控制类")
public class UserInfoController {
	
	@Resource
	private UserInfoService userInfoService;

	@Resource
	private OrgInfoService orgInfoService;

	@Resource
	private UserRoleService userRoleService;

	@Resource
	private RoleInfoService roleInfoService;
	
	@Autowired
	private RedisUtil redisUtil;
	
	@Autowired
	private TUrlConfigService tUrlConfigService;
	
	@Resource
	private UserOrgService userOrgService;

	@Value("${web.upload-path}")
	private String uploadPath;
	
	@Value("${localOrCloud}")
	private String localOrCloud;
	
	@Value("${forTheConnection}")
	private String forTheConnection;
	
	@Value("${loginTime}")
	private String loginTime;
	
	@Value("${FLATID}")
	private String FLATID;
	
	@Value("${KEY}")
	private String KEY;
	
	@Value("${wsdlUrl}")
	private String wsdlUrl;

	/**
	 * 查找用户信息
	 * @param pageNum	页码
	 * @param pageSize	每页条数
	 * @return
	 */
	@RequestMapping(value="/getUser", method = RequestMethod.GET)
	@ApiOperation(value="查询用户信息", notes = "查询用户信息列表，支持分页")
	@ApiImplicitParams({
			@ApiImplicitParam(name = "pageNum", value = "页码", paramType = "query"),
			@ApiImplicitParam(name = "pageSize", value = "每页条数", paramType = "query"),
			@ApiImplicitParam(name = "username", value = "用户名", paramType = "query"),
			@ApiImplicitParam(name = "realName", value = "用户真实姓名", paramType = "query")
	})
	public PageInfo<UserInfo> getUserInfo(HttpServletRequest request, Integer pageNum, Integer pageSize,
                                          String username, String realName, Integer roleId, String orgIds) {
		
		if (pageNum!=null && pageSize!=null) {
			PageHelper.startPage(pageNum, pageSize);
		}
		
		HashMap<Object, Object> map = new HashMap<Object, Object>();
		map.put("username", username);
		map.put("realName", realName);
		map.put("roleId", roleId);
		List<UserInfo> listAll=userInfoService.queryAll(map);
		
		if("正常".equals(NetStateUtil.connectingAddress(wsdlUrl))) {
			try {
				ExchangeServer wb = new ExchangeServer();
				IExchangeService action = wb.getExchangeServiceImplPort();
				for(UserInfo userInfo:listAll) {
					String roleNames = userInfo.getRoleNames();
					String userRolesInfo = action.getUserRolesInfo(FLATID, KEY, Integer.toString(userInfo.getUserId()) );
					JSONObject obj = JSONObject.parseObject(userRolesInfo);  
					String result = obj.getString("result");
					JSONArray data = JSONArray.parseArray(result);
					for(int i=0;i<data.size();i++) {
						Map<String,Object> maps = (Map<String,Object>)data.get(i);
						if(roleNames != null && !roleNames.isEmpty()) {
							roleNames = roleNames + "," + ObjectUtils.toString(maps.get("roleName"),"");
						}else {
							roleNames = ObjectUtils.toString(maps.get("roleName"),"");
						}
					}
					userInfo.setRoleNames(roleNames);
				}
			} catch (Exception e) {
				e.printStackTrace();
			}
		}
		
		PageInfo<UserInfo> pageInfo = new PageInfo<UserInfo>(listAll);
		return pageInfo;
	}
	
	/**
	 * 查找用户信息
	 * @param pageNum	页码
	 * @param pageSize	每页条数
	 * @return
	 */
	@RequestMapping(value="/getUserInfo", method = RequestMethod.GET)
	@ApiOperation(value="查询用户信息", notes = "查询用户信息列表，支持分页")
	@ApiImplicitParams({
		@ApiImplicitParam(name = "pageNum", value = "页码", paramType = "query"),
		@ApiImplicitParam(name = "pageSize", value = "每页条数", paramType = "query"),
		@ApiImplicitParam(name = "username", value = "用户名", paramType = "query"),
		@ApiImplicitParam(name = "realName", value = "用户真实姓名", paramType = "query")
	})
	public PageInfo<UserInfo> getUser(HttpServletRequest request, Integer pageNum, Integer pageSize) {
		
		if (pageNum!=null && pageSize!=null) {
			PageHelper.startPage(pageNum, pageSize);
		}
		HashMap<Object, Object> map = new HashMap<Object, Object>();
		List<UserInfo> listAll=userInfoService.queryAll(map);
		PageInfo<UserInfo> pageInfo = new PageInfo<UserInfo>(listAll);
		return pageInfo;
	}

	@RequestMapping(value="/checkRepeat", method = RequestMethod.GET)
	@ApiOperation(value="检查同一个库下相同身份证号的人是否已存在", notes = "")
	@ApiImplicitParams({
		@ApiImplicitParam(name = "orgId", value = "组织机构编码", paramType = "query"),
		@ApiImplicitParam(name = "sfzhm", value = "身份证号", paramType = "query"),
		@ApiImplicitParam(name = "userId", value = "用户ID", paramType = "query"),
		@ApiImplicitParam(name = "type", value = "检查类型", paramType = "query")
	})
	public String checkRepeat(Integer orgId, String sfzhm, Integer userId, String type) {
		String flag = "false";
		try {
			flag = userInfoService.checkRepeat(orgId, sfzhm, userId, type);
		} catch (Exception e) {
			e.printStackTrace();
		}
		return flag;
	}

	/**
	 * 查找审批用户信息
	 * @param roleName	角色名称
	 * @param orgId	组织机构编码
	 * @return
	 */
	@RequestMapping(value="/getAuditUser", method = RequestMethod.GET)
	@ApiOperation(value="查询用户信息", notes = "查询用户信息列表，支持分页")
	@ApiImplicitParams({
			@ApiImplicitParam(name = "roleName", value = "角色名称", paramType = "query"),
			@ApiImplicitParam(name = "orgId", value = "组织机构编码", paramType = "query")
	})
	public List<Map<Object, Object>> getAuditUserInfo(String roleName, Integer orgId) {
		Map<Object, Object> map = new HashMap<>();
		if (StringUtils.isNotBlank(roleName) && orgId != null) {
			map.put("roleName", roleName);
			map.put("orgId", orgId);
		}
		List<Map<Object, Object>> listMap = userInfoService.queryAuditUser(map);
		return listMap;
	}

	/**
	 * 用户登录
	 * @param request
	 * @param username	用户名
	 * @param password	密码
	 * @return
	 */
	@RequestMapping(value="/login", method = RequestMethod.POST)
	@ApiOperation(value="用户登录", notes = "用户登录")
	@ApiImplicitParams({
			@ApiImplicitParam(name = "username", value = "用户名", paramType = "form"),
			@ApiImplicitParam(name = "password", value = "密码", paramType = "form"),
			@ApiImplicitParam(name = "yzm", value = "验证码", paramType = "form"),
			@ApiImplicitParam(name = "from", value = "来源，app代表来自于移动端", paramType = "form")
	})
	public Map<String, Object> Login(HttpServletRequest request, String username, String password, String yzm, String from) {
		Map<String, Object> map = new HashMap<String, Object>();
		if (StringUtils.isBlank(from) || from.compareTo("app")!=0) {
			String yzm_code = ObjectUtils.toString(request.getSession().getAttribute("Service_Code"),"");
			if (!yzm_code.equals(yzm)) {
				map.put("status", false);
				map.put("success", "验证码输入错误！");
				return map;
			}
		}
		UserInfo userInfo = null;
		if(!userInfoService.validUsername(null,username)) {//用户名存在
			/**************************限制单个用户的同时重复登录操作************************/
			String sessionId = ObjectUtils.toString(redisUtil.get(username+"_sessinId"),"");
			if(!sessionId.equals("")) {
				HttpSession session = MySessionContext.getSession(sessionId);
				//当第二次登录时，把第一个session剔除
                if(session != null){
                	try {
                		ObjectMapper mapper = new ObjectMapper();
                		String userInfoObject = ObjectUtils.toString(session.getAttribute("userInfo"),"");
                		userInfo = mapper.readValue(userInfoObject, UserInfo.class);
                	}catch(Exception e) {
                		e.printStackTrace();
                	}
                	if(username.equals(userInfo.getUsername())) {
                		session.invalidate();//强制销毁session
                		MySessionContext.DelSession(session);
                	}
                }
			}
            /**************************限制单个用户的同时重复登录操作************************/
			
			/*String phoneNumber = userInfoService.queryPhoneByusername(username);
			String verify = ObjectUtils.toString(redisUtil.get(phoneNumber),"");
			String verifyCode = ObjectUtils.toString(request.getParameter("verifyCode"),"");
			if(verify.equals(verifyCode)) {*/
			// 把用MD5加密过的密码再用SHA1的方式加密一次，然后用SHA1加密后的密码去数据库中保存的密码做检索
			String new_password = SHA1Util.encode(password);
			userInfo = userInfoService.valid(username, new_password);
			/*}else {
				map.put("status", false);
				map.put("success", "手机验证码错误或者已过期！");
				return map;
			}*/
			
			int limitTime = Integer.parseInt(loginTime);
			/**************************用户登录失败处理功能，限制用户非法登录次数************************/
			if(userInfo == null) {//密码输入错误
				map.put("status", false);
				int errorCount = Integer.parseInt(ObjectUtils.toString(redisUtil.get(username+"_errorCount"),"0"))+1 ;
				map.put("success", "密码输入错误"+errorCount+"次，错误5次，将限制登录！");
				if(errorCount == 5) {//密码输入次数等于5次，将限制登录5分钟
					redisUtil.set(username+"_time", String.valueOf(System.currentTimeMillis()));//保存最后一次错误时间戳
					map.put("success", "密码输入错误"+errorCount+"次，请"+limitTime+"分钟后再次登录！");
					//倒计时5分钟，清空密码输入错误次数
					final Timer timer=new Timer();
				    timer.schedule(new TimerTask() {
				        @Override
				        public void run() {
				        	redisUtil.set(username+"_errorCount", "0");
				        	timer.cancel();
				        }
				    },limitTime*60*1000);
				}else if(errorCount > 5){//大于5次不再进行时间记录，直接返回需要等待的时间
					String errorTime = ObjectUtils.toString(redisUtil.get(username+"_time"));
					Long s = (System.currentTimeMillis() - Long.parseLong(errorTime)) / (1000 * 60);
					map.put("success", "请"+(limitTime-s)+"分钟后再次登录！");
				}
				redisUtil.set(username+"_errorCount", String.valueOf(errorCount));
			}else {//登录成功
				//获取登录错误次数
				String errorCount = ObjectUtils.toString(redisUtil.get(username+"_errorCount"),"0");
				if(Integer.parseInt(errorCount) < 5) {//错误次数小于5次，清空错误登录次数
					redisUtil.set(username+"_errorCount", "0");
				}else{//错误次数大于等于5次，验证时间是否过5分钟
					String errorTime = ObjectUtils.toString(redisUtil.get(username+"_time"));
					Long s = (System.currentTimeMillis() - Long.parseLong(errorTime)) / (1000 * 60);
					if(s > limitTime) {//时间超过5分钟，允许登录并且清空错误次数
						redisUtil.set(username+"_errorCount", "0");
					}else {
						userInfo = null;
						map.put("success", "请"+(limitTime-s)+"分钟后再次登录！");
					}
				}
			}
			/**************************用户登录失败处理功能，限制用户非法登录次数************************/
			
		}else {
			map.put("status", false);
			map.put("success", "该用户名不存在，登录失败！");
		}
		
		if (userInfo != null) {
			List<String> tableList = userInfoService.queryUserBusiness();
			redisUtil.del("tableList");
			redisUtil.lSet("tableList", tableList);
			redisUtil.set(username+"_sessinId", request.getSession().getId());
			//把对象转变成一个json
			ObjectMapper mapper = new ObjectMapper();
			try {
				String userInfoObject = mapper.writeValueAsString(userInfo);
				HttpSession session = request.getSession();
				MySessionContext.AddSession(session);//将当前session存到静态map里
				// 当前登录用户
				session.setAttribute("userInfo", userInfoObject);
				// 当前登录用户id
				session.setAttribute("userId", userInfo.getUserId());
				// 当前登录用户机构id
				session.setAttribute("orgId", userInfo.getOrgId());
				// 当前组织信息
				OrgInfo orgInfo = orgInfoService.findById(userInfo.getOrgId());
				String orgInfoObject = mapper.writeValueAsString(orgInfo);
				session.setAttribute("orgInfo", orgInfoObject);
				// 当前用户所属的库信息及库编号
				OrgInfo depotInfo = orgInfoService.getDepotInfo(orgInfo);
				if(depotInfo!=null){
					String depotInfoObject = mapper.writeValueAsString(depotInfo);
					session.setAttribute("depotInfo", depotInfoObject);
					session.setAttribute("depotId", depotInfo.getOrgId());
				}
			} catch (JsonProcessingException e) {
				e.printStackTrace();
			}
			map.put("status", true);
			map.put("userInfo", userInfo);
		}
		return map;
	}

	/**
	 * 获取用户登录信息,包括当前用户、当前组织、当前库、当前库编号
	 * @param request
	 * @return
	 */
	@RequestMapping(value="/getLoginInfo", method = RequestMethod.GET)
	@ApiOperation(value="获取用户登录信息", notes = "获取用户登录信息")
	public Map<String, Object> getLoginInfo(HttpServletRequest request) {
		Map<String, Object> modelMap = new HashMap<>();
		try {
			// JSON字符串转对象
			ObjectMapper mapper = new ObjectMapper();
			String userInfoObject = ObjectUtils.toString(request.getSession().getAttribute("userInfo"),"");
			if(!userInfoObject.isEmpty()) {
				UserInfo userInfo = mapper.readValue(userInfoObject, UserInfo.class);
				//获取当前用户的所在session的登录信息
				/*String sessionId = ObjectUtils.toString(redisUtil.get(userInfo.getUsername()+"_sessinId"),"");
				if(!sessionId.equals("")) {
					HttpSession session = MySessionContext.getSession(sessionId);
					//判断前时间跟最近一次操作时间相差的时间是否超过15分钟
	                if(session != null){
	                	Instant this_instant = new Date().toInstant();
	            		LocalDateTime this_DateTime = LocalDateTime.ofInstant(this_instant, java.time.ZoneId.systemDefault());
	            		
	            		//获取上次操作时间
	            		Instant last_instant = ParameterUtil.string2datetime(ObjectUtils.toString(request.getSession().getAttribute("lastOperateTime"),"")).toInstant();
	            		LocalDateTime last_DateTime = LocalDateTime.ofInstant(last_instant, java.time.ZoneId.systemDefault());

	                    // 计算时间差
	                    Duration duration = Duration.between(last_DateTime, this_DateTime);
	                 
	                    // 判断是否大于15分钟
	                    if (duration.toMinutes() > 30) {
	                        return modelMap;
	                    }
	                }
				}*/
				
				//UserInfo userInfo = (UserInfo)request.getSession().getAttribute("userInfo");
				modelMap.put("userInfo", userInfo);
			}
			String orgInfoObject = ObjectUtils.toString(request.getSession().getAttribute("orgInfo"),"");
			if(!orgInfoObject.isEmpty()) {
				OrgInfo orgInfo = mapper.readValue(orgInfoObject, OrgInfo.class);
				//OrgInfo orgInfo = (OrgInfo)request.getSession().getAttribute("orgInfo");
				modelMap.put("orgInfo", orgInfo);
			}
			String depotInfoObject = ObjectUtils.toString(request.getSession().getAttribute("depotInfo"),"");
			if(!depotInfoObject.isEmpty()) {
				OrgInfo depotInfo = mapper.readValue(depotInfoObject, OrgInfo.class);
				//OrgInfo depotInfo = (OrgInfo)request.getSession().getAttribute("depotInfo");
				modelMap.put("depotInfo", depotInfo);
			}
			Integer depotId = (Integer)request.getSession().getAttribute("depotId");
			modelMap.put("depotId", depotId);
		} catch (IOException e) {
			e.printStackTrace();
		}
		
		return modelMap;
	}
	
	/**
	 * 退出登录
	 * @param request
	 * @return
	 */
	@RequestMapping(value="/exitLogin", method = RequestMethod.GET)
	@ApiOperation(value="退出登录", notes = "退出登录，删除session")
	public Map<String, Object> exitLogin(HttpServletRequest request) {
		Map<String, Object> modelMap = new HashMap<>();
		request.getSession().removeAttribute("userInfo");
		request.getSession().removeAttribute("orgInfo");
		request.getSession().removeAttribute("depotInfo");
		request.getSession().removeAttribute("orgId");
		request.getSession().removeAttribute("depotId");
		request.getSession().invalidate();//强制销毁session
		MySessionContext.DelSession(request.getSession());
		modelMap.put("status", "success");
		return modelMap;
	}

	/**
	 * 验证用户名是否可用
	 * @param userId	用户编号
	 * @param username	用户名
	 * @return
	 */
	@RequestMapping(value="/validUsername", method = RequestMethod.GET)
	@ApiOperation(value="验证用户名", notes = "验证用户名是否可用")
	@ApiImplicitParams({
			@ApiImplicitParam(name = "userId", value = "用户编号", paramType = "query"),
			@ApiImplicitParam(name = "username", value = "用户名", paramType = "query")
	})
	public Boolean validUsername(Integer userId, String username) {
		return userInfoService.validUsername(userId, username);
	}
	
	/**
	 * 通过用户名查询手机号
	 * @param username	用户名
	 * @return
	 */
	@RequestMapping(value="/queryPhoneByusername", method = RequestMethod.GET)
	@ApiOperation(value="查询手机号", notes = "通过用户名查询手机号")
	@ApiImplicitParams({
		@ApiImplicitParam(name = "username", value = "用户名", paramType = "query")
	})
	public Map<String,Object> queryPhoneByusername(String username) {
		Map<String,Object> map = new HashMap<String,Object>();
		String phone = userInfoService.queryPhoneByusername(username);
		map.put("phone", phone);
		return map;
	}
	
	/**
	 * 验证手机号是否存在
	 * @param phoneNumber	手机号
	 * @return
	 */
	@RequestMapping(value="/validPhone", method = RequestMethod.GET)
	@ApiOperation(value="验证手机号", notes = "验证手机号是否存在")
	@ApiImplicitParams({
		@ApiImplicitParam(name = "phoneNumber", value = "手机号", paramType = "query"),
		@ApiImplicitParam(name = "userId", value = "用户编号", paramType = "query")
	})
	public Boolean validPhone(String phoneNumber, Integer userId) {
		boolean flag = false;
		String username = userInfoService.validPhone(phoneNumber,userId);
		if(!username.equals("")) {
			flag = true;
		}
		return flag;
	}
	
	/**
	 * 验证用户输入的原密码是否正确
	 * @param oldPassword	用户输入的原密码
	 * @return
	 */
	@RequestMapping(value="/validPassword", method = RequestMethod.GET)
	@ApiOperation(value="验证用户输入的原密码是否正确", notes = "验证用户输入的原密码是否正确")
	@ApiImplicitParams({
			@ApiImplicitParam(name = "oldPassword", value = "用户密码", paramType = "query")
	})
	public boolean validPassword(HttpServletRequest request, String oldPassword) throws Exception{
		ObjectMapper mapper = new ObjectMapper();
		String userInfoObject = ObjectUtils.toString(request.getSession().getAttribute("userInfo"),"");
		UserInfo userInfo = mapper.readValue(userInfoObject, UserInfo.class);
		String resPassword = userInfo.getPassword();
		return userInfoService.validPassword(resPassword, oldPassword);
	}

	/**
	 * 重置密码
	 * @param userId
	 * @return
	 */
	@RequestMapping(value="/resetPassword", method = RequestMethod.POST)
	@ApiOperation(value="重置密码", notes = "重置密码")
	@ApiImplicitParams({
			@ApiImplicitParam(name = "userId", value = "用户编号", paramType = "form"),
			@ApiImplicitParam(name = "userPwd", value = "用户密码", paramType = "form")
	})
	public void resetPassword(Integer userId, String userPwd) {
		userInfoService.resetPassword(userId,userPwd);
	}
	
	/**
	 * 根据编号查找用户信息
	 * @param userId 用户编号
	 * @return	UserInfo	用户信息
	 */
	@RequestMapping(value="/edit", method = RequestMethod.GET)
	@ApiOperation(value="编辑用户信息", notes = "编辑用户信息，用于新增或者修改")
	@ApiImplicitParams({
			@ApiImplicitParam(name = "userId", value = "用户编号", paramType = "query")
	})
	public UserInfo edit(Integer userId) {
		UserInfo userInfo = new UserInfo();
		if (userId != null) {
			userInfo = userInfoService.findById(userId);
		}
		if("正常".equals(NetStateUtil.connectingAddress(wsdlUrl))) {
			try {
				List<String> roleIds = userInfo.getRoleIds();
				ExchangeServer wb = new ExchangeServer();
				IExchangeService action = wb.getExchangeServiceImplPort();
				String userRolesInfo = action.getUserRolesInfo(FLATID, KEY, Integer.toString(userId) );
				JSONObject obj = JSONObject.parseObject(userRolesInfo);  
				String result = obj.getString("result");
				JSONArray data = JSONArray.parseArray(result);
				for(int i=0;i<data.size();i++) {
					Map<String,Object> map = (Map<String,Object>)data.get(i);
					roleIds.add(ObjectUtils.toString(map.get("roleId"),""));
				}
				userInfo.setRoleIds(roleIds);
			} catch (Exception e) {
				e.printStackTrace();
			}
		}
		return userInfo;
	}

	/**
	 * 保存用户信息
	 * @param userInfoJson	用户信息json字符串
	 * @return
	 * @throws JsonParseException
	 * @throws JsonMappingException
	 * @throws IOException
	 */
	@RequestMapping(value="/save", method = RequestMethod.POST)
	@ApiOperation(value="保存用户", notes = "保存用户信息，新增或者修改")
	@ApiImplicitParams({
			@ApiImplicitParam(name = "userInfoJson", value = "用户信息JSON串", paramType = "form"),
			@ApiImplicitParam(name = "userRoleIds", value = "用户角色编号，多个之间用逗号隔开", paramType = "form"),
			@ApiImplicitParam(name = "crkRoleIds", value = "出入库角色编号，多个之间用逗号隔开", paramType = "form")
	})
	public Map<String, Object> save(String userInfoJson , String userRoleIds, String crkRoleIds) {
		Map<String, Object> modelMap = new HashMap<>();
		try {
			modelMap = userInfoService.save(userInfoJson, userRoleIds, crkRoleIds, wsdlUrl, FLATID, KEY);
		} catch (Exception e) {
			modelMap.put("status", "error");
			e.printStackTrace();
		}

		return modelMap;
	}
	
	/**
	 * 根据编号删除用户编纂 
	 * @param userId	用户编号
	 * @return
	 */
	@RequestMapping(value="/remove", method = RequestMethod.POST)
	@ApiOperation(value="删除用户", notes = "按用户编号删除用户")
	@ApiImplicitParams({
			@ApiImplicitParam(name = "userId", value = "用户名", paramType = "form")
	})
	public Map<String, Object> remove(Integer userId) {
		Map<String, Object> modelMap = new HashMap<>();
		if (userId != null) {
			try {
				userInfoService.deleteAll(userId);
			} catch (Exception e) {
				e.printStackTrace();
			}
		}
		modelMap.put("status", "success");
		return modelMap;
	}

	@RequestMapping(value = "/modifyPwd", produces = MediaType.APPLICATION_JSON_VALUE, method = RequestMethod.POST)
	@ApiOperation(value="修改密码")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "oldPassword", value = "原密码", paramType = "form"),
            @ApiImplicitParam(name = "newPassword", value = "新密码", paramType = "form")
    })
	public UserInfo modifyPwd(HttpServletRequest request, String oldPassword, String newPassword) {
		try {
			System.out.println("oldPassword："+oldPassword);
			System.out.println("newPassword："+newPassword);
			ObjectMapper mapper = new ObjectMapper();
			String userInfoObject = ObjectUtils.toString(request.getSession().getAttribute("userInfo"),"");
			UserInfo userInfo = mapper.readValue(userInfoObject, UserInfo.class);
			String old_password = SHA1Util.encode(userInfoService.getMD5(oldPassword));
			System.out.println("加密后的原密码："+old_password);
			System.out.println("userInfo.getPassword()："+userInfo.getPassword());
			System.out.println(userInfo.getPassword().equals(old_password));
			String pwd = userInfo.getPassword();
			if (userInfo.getPassword().equals(old_password)) {
				String md5_pwd = userInfoService.getMD5(newPassword);
				String sha1_pwd = SHA1Util.encode(md5_pwd);
				System.out.println("加密后的新密码："+sha1_pwd);
				userInfo.setPassword(sha1_pwd);
				userInfo.setUpdateTime(new Date());
				//userInfo.setPassword(userInfoService.getMD5(newPassword));
				userInfoService.update(userInfo);
				
				String userInfoObjectNew = mapper.writeValueAsString(userInfo);
				request.getSession().setAttribute("userInfo", userInfoObjectNew);
				return userInfo;
			}
		} catch (IOException e) {
			e.printStackTrace();
		}
		return null;
	}

	/**
	 * 查询当前库下所有的用户
	 * @param orgId
	 * @return
	 */
	@RequestMapping(value="/findAllUserByOrgId", method = RequestMethod.GET)
	@ApiOperation(value="查询当前库下所有的用户", notes = "查询当前库下所有的用户")
	@ApiImplicitParams({
			@ApiImplicitParam(name = "orgId", value = "单位ID", paramType = "form")
	})
	public List<UserInfo> findAllUserByOrgId(Integer orgId){
		UserInfoExample example = new UserInfoExample();
		Criteria criteria = example.createCriteria();
		List<UserInfo> list = null;
		try {
			// 拼接查询条件单位
			if (null != orgId && !"".equals(orgId)) {
				criteria.andOrgIdEqualTo(orgId);
			}
			//criteria.andDelFlagEqualTo(1); // 查询正常数据（暂时不做限制）
			list = userInfoService.queryByExample(example);
		} catch (Exception e) {
			e.printStackTrace();
		}
		return list;
	}

    @RequestMapping(value="/findAllUserMapByOrgId", method = RequestMethod.GET)
    @ApiOperation(value="查询当前库下所有的用户枚举，返回id与用户姓名的map", notes = "")
    public Map<Integer, Object> findAllUserMapByOrgId(HttpServletRequest request) {
        Map<Integer, Object> map = new HashMap<>();
		try {
			ObjectMapper mapper = new ObjectMapper();
			String depotInfoObject = ObjectUtils.toString(request.getSession().getAttribute("depotInfo"),"");
			if(!depotInfoObject.isEmpty()) {
				OrgInfo orgInfo = mapper.readValue(depotInfoObject, OrgInfo.class);
				
				UserInfoExample keeperExample = new UserInfoExample();
				Criteria criteria = keeperExample.createCriteria();
				List<UserInfo> list = null;
				try {
					Integer orgId = null;
					if (orgInfo != null) {
						orgId=orgInfo.getOrgId();//所属库id
					}
					// 拼接查询条件单位
					if (null != orgId && !"".equals(orgId)) {
						criteria.andOrgIdEqualTo(orgId);
					}
					list = userInfoService.queryByExample(keeperExample);
					for (UserInfo userInfo : list) {
						map.put(userInfo.getUserId(), userInfo.getRealName());
					}
				} catch (Exception e) {
					e.printStackTrace();
				}
			}
		} catch (IOException e1) {
			e1.printStackTrace();
		}
        
        return map;
    }

    /**
	 * 生成验证码图片, 并将验证码图片上传到服务器，然后将生成的验证码串值保存到session 中
	 * 
	 * @param request
	 **/
	@RequestMapping(value="/getCode", produces = MediaType.APPLICATION_JSON_VALUE, method = RequestMethod.GET)
	@ApiOperation(value="生成验证码图片", notes = "生成验证码图片")
	public Map<String, Object> getCode(HttpServletRequest request, Integer fontSize, Integer width, Integer heigth) {

		// 图片流
		BufferedImage image;

		// 获取验证码数字字符串
		String yzm_code = SecurityCode.getSecurityCode(4, SecurityCodeLevel.Hard, false).toLowerCase();
		request.getSession().setAttribute("Service_Code", yzm_code);
		// 生成验证码图片
		image = SecurityImage.createImage(yzm_code, fontSize, width, heigth);

		ByteArrayOutputStream baos = new ByteArrayOutputStream();
        try {
			ImageIO.write(image, "jpg", baos);
		} catch (IOException e1) {
			e1.printStackTrace();
		}

        //将图片转换为base64位二进制编码返回
        String imgBase64 = new Base64().encodeToString(baos.toByteArray());
  		Map<String, Object> map = new HashMap<String, Object>();
  		map.put("file", imgBase64);
  		return map;
	}
	
	/**
     * 判断当前环境是外网还是库内网
     * @param request
     * @return
     */
    @RequestMapping(value="/environmentalJudgement", method = RequestMethod.GET)
    @ApiOperation(value="判断当前访问环境", notes = "")
    public Map<String, Object> environmentalJudgement(HttpServletRequest request) {
    	Map<String, Object> map = new HashMap<String, Object>();
    	map.put("localOrCloud", localOrCloud);//判断是云端还是库端
    	map.put("displayType", localOrCloud);//云端库端页面菜单展示控制
    	if(localOrCloud.equals("1")) {//1云端，2本地,只有云端的时候库级人员会跳转
    		try {
    			String orgId = ObjectUtils.toString(request.getSession().getAttribute("orgId"),"");
    			if(!orgId.isEmpty()) {
    				Map<String, Object> paramMap = new HashMap<String, Object>();
    				paramMap.put("orgId", orgId);
    				paramMap.put("userType", "1");//只查询库级的
    				List<TUrlConfig> list = tUrlConfigService.queryByExample(paramMap);
    				String depotIp = "";
					for(TUrlConfig enty:list) {
						depotIp = ObjectUtils.toString(enty.getIntranetUrl(),"");
						break;
					}
					String ip = getIpAddr(request);
					if(!depotIp.equals("") && ip.substring(0,6).equals(NetStateUtil.getIpFromUrl(depotIp).substring(0,6))
							&& "正常".equals(NetStateUtil.connectingAddress(depotIp+"/intelligents/station/getStations?orgId=" + orgId))) {//当前ip是库内ip段的说明是库内访问
						map.put("depotUrl", depotIp);
						map.put("localOrCloud", 2);//访问应急端的时候直接设置为本地，不然智能仓房操作按钮出不来
					}
					/*if(!depotIp.equals("") && "正常".equals(NetStateUtil.connectingAddress(depotIp+"/intelligents/station/getStations"))) {
						map.put("depotUrl", depotIp);
						map.put("localOrCloud", 2);//访问应急端的时候直接设置为本地，不然智能仓房操作按钮出不来
					};*/ 
    			}
    		} catch (Exception e) {
    			e.printStackTrace();
    		}
    	}
        return map;
    }

    @RequestMapping(value="/forTheConnection", method = RequestMethod.GET)
    @ApiOperation(value="传输配置文件里边的出入库连接", notes = "")
    public Map<String, Object> forTheConnection(HttpServletRequest request) {
       Map<String, Object> map = new HashMap<String, Object>();
       map.put("forTheConnection", forTheConnection);//传输配置文件里边的出入库连接
       map.put("RemoteLogin", "true");//代表异地登录
       if(localOrCloud.equals("1")) {//1云端，2本地,只有云端的时候出入库人员会跳转
           try {
              String orgId = ObjectUtils.toString(request.getSession().getAttribute("orgId"),"");
              if(!orgId.isEmpty()) {
                 Map<String, Object> paramMap = new HashMap<String, Object>();
                 paramMap.put("orgId", orgId);
                 paramMap.put("userType", "2");//只查询出入库的
                 List<TUrlConfig> list = tUrlConfigService.queryByExample(paramMap);
                 String depotIp = "";
                for(TUrlConfig enty:list) {
                   depotIp = ObjectUtils.toString(enty.getIntranetUrl(),"");
                   break;
                }
                String ip = getIpAddr(request);
                if(!depotIp.equals("") && ip.substring(0,6).equals(NetStateUtil.getIpFromUrl(depotIp).substring(0,6)) 
                		&& "正常".equals(NetStateUtil.connectingAddress(depotIp))) {//当前ip是库内ip段的说明是库内访问
                   map.put("forTheConnection", depotIp);
                   map.put("RemoteLogin", "false");//代表本地登录
                   int index = depotIp.indexOf("Base");
                   String crk_url = depotIp.substring(0, index-1);
                   map.put("crk_url", crk_url);
                } else {
                	int index = forTheConnection.indexOf("Base");
                    String crk_url = forTheConnection.substring(0, index-1);
                    map.put("crk_url", crk_url);
                }
                /*if(!depotIp.equals("") && "正常".equals(NetStateUtil.connectingAddress(depotIp))) {
                   map.put("forTheConnection", depotIp);
                };*/ 
              }
           } catch (Exception e) {
              e.printStackTrace();
           }
        }
       return map;
    }

	/**
	 * 查找行政机构用户信息
	 * @param pageNum	页码
	 * @param pageSize	每页条数
	 * @return
	 */
	@RequestMapping(value="/getUserByOrgClass", method = RequestMethod.GET)
	@ApiOperation(value="查询行政机构用户信息", notes = "查询行政机构用户信息列表，支持分页")
	@ApiImplicitParams({
			@ApiImplicitParam(name = "pageNum", value = "页码", paramType = "query"),
			@ApiImplicitParam(name = "pageSize", value = "每页条数", paramType = "query"),
			@ApiImplicitParam(name = "orgClassId", value = "行政机构编码", paramType = "query")
	})
	public PageInfo<UserInfo> getUserByOrgClass(HttpServletRequest request, Integer pageNum, Integer pageSize, Integer orgClassId) {
		
		if (pageNum!=null && pageSize!=null) {
			PageHelper.startPage(pageNum, pageSize);
		}
		
		Map<Object, Object> map = new HashMap<Object, Object>();
		map.put("orgClassId", orgClassId);
		List<UserInfo> list=userInfoService.queryAllByOrgClass(map);
		PageInfo<UserInfo> pageInfo = new PageInfo<UserInfo>(list);
		return pageInfo;
	}
	
	/**
	 * 获取IP地址
	 * 
	 * 使用Nginx等反向代理软件， 则不能通过request.getRemoteAddr()获取IP地址
	 * 如果使用了多级反向代理的话，X-Forwarded-For的值并不止一个，而是一串IP地址，X-Forwarded-For中第一个非unknown的有效IP字符串，则为真实IP地址
	 */
	public static String getIpAddr(HttpServletRequest request) {

		String ip = request.getHeader("x-forwarded-for");
		if (ip == null || ip.length() == 0 || "unknown".equalsIgnoreCase(ip)) {
			ip = request.getHeader("Proxy-Client-IP");
		}
		if (ip == null || ip.length() == 0 || "unknown".equalsIgnoreCase(ip)) {
			ip = request.getHeader("WL-Proxy-Client-IP");
		}
		if (ip == null || ip.length() == 0 || "unknown".equalsIgnoreCase(ip)) {
			ip = request.getRemoteAddr();
		}
		return "0:0:0:0:0:0:0:1".equals(ip) ? "127.0.0.1" : ip;
	}
	

	/**
	 * 市平台跳转库平台首页 用户登录
	 * @param request
	 * @param kqbm	库区编码
	 * @return
	 */
	@RequestMapping(value="/loginspt", method = RequestMethod.POST)
	@ApiOperation(value="用户登录", notes = "用户登录")
	@ApiImplicitParams({
		    @ApiImplicitParam(name = "kqbm", value = "库区编码", paramType = "query")
	})
	public Map<String, Object> LoginSpt(HttpServletRequest request, @RequestParam("kqbm") String kqbm) {
			
		Map<Object, Object> map1 = new HashMap<>();
		if (StringUtils.isNotBlank(kqbm) && kqbm != null && kqbm != "") {
			map1.put("kqbm", kqbm);
		}
		List<Map<Object, Object>> listMap = userInfoService.sptgetUserInfo(map1);
		String username = (String) listMap.get(0).get("username");
		String password = (String) listMap.get(0).get("password");
		
		Map<String, Object> map = new HashMap<String, Object>();
		UserInfo userInfo = null;
		if(!userInfoService.validUsername(null,username)) {//用户名存在
			/**************************限制单个用户的同时重复登录操作************************/
			String sessionId = ObjectUtils.toString(redisUtil.get(username+"_sessinId"),"");
			if(!sessionId.equals("")) {
				HttpSession session = MySessionContext.getSession(sessionId);
				//当第二次登录时，把第一个session剔除
                if(session != null){
                	try {
                		ObjectMapper mapper = new ObjectMapper();
                		String userInfoObject = ObjectUtils.toString(session.getAttribute("userInfo"),"");
                		userInfo = mapper.readValue(userInfoObject, UserInfo.class);
                	}catch(Exception e) {
                		e.printStackTrace();
                	}
                	if(username.equals(userInfo.getUsername())) {
                		session.invalidate();//强制销毁session
                		MySessionContext.DelSession(session);
                	}
                }
			}
            /**************************限制单个用户的同时重复登录操作************************/
			userInfo = userInfoService.valid(username, password);
			int limitTime = Integer.parseInt(loginTime);
			/**************************用户登录失败处理功能，限制用户非法登录次数************************/
			if(userInfo == null) {//密码输入错误
				map.put("status", false);
				int errorCount = Integer.parseInt(ObjectUtils.toString(redisUtil.get(username+"_errorCount"),"0"))+1 ;
				map.put("success", "密码输入错误"+errorCount+"次，错误5次，将限制登录！");
				if(errorCount == 5) {//密码输入次数等于5次，将限制登录5分钟
					redisUtil.set(username+"_time", String.valueOf(System.currentTimeMillis()));//保存最后一次错误时间戳
					map.put("success", "密码输入错误"+errorCount+"次，请"+limitTime+"分钟后再次登录！");
					//倒计时5分钟，清空密码输入错误次数
					final Timer timer=new Timer();
				    timer.schedule(new TimerTask() {
				        @Override
				        public void run() {
				        	redisUtil.set(username+"_errorCount", "0");
				        	timer.cancel();
				        }
				    },limitTime*60*1000);
				}else if(errorCount > 5){//大于5次不再进行时间记录，直接返回需要等待的时间
					String errorTime = ObjectUtils.toString(redisUtil.get(username+"_time"));
					Long s = (System.currentTimeMillis() - Long.parseLong(errorTime)) / (1000 * 60);
					map.put("success", "请"+(limitTime-s)+"分钟后再次登录！");
				}
				redisUtil.set(username+"_errorCount", String.valueOf(errorCount));
			}else {//登录成功
				//获取登录错误次数
				String errorCount = ObjectUtils.toString(redisUtil.get(username+"_errorCount"),"0");
				if(Integer.parseInt(errorCount) < 5) {//错误次数小于5次，清空错误登录次数
					redisUtil.set(username+"_errorCount", "0");
				}else{//错误次数大于等于5次，验证时间是否过5分钟
					String errorTime = ObjectUtils.toString(redisUtil.get(username+"_time"));
					Long s = (System.currentTimeMillis() - Long.parseLong(errorTime)) / (1000 * 60);
					if(s > limitTime) {//时间超过5分钟，允许登录并且清空错误次数
						redisUtil.set(username+"_errorCount", "0");
					}else {
						userInfo = null;
						map.put("success", "请"+(limitTime-s)+"分钟后再次登录！");
					}
				}
			}
			/**************************用户登录失败处理功能，限制用户非法登录次数************************/
			
		}else {
			map.put("status", false);
			map.put("success", "该用户名不存在，登录失败！");
		}
		
		if (userInfo != null) {
			List<String> tableList = userInfoService.queryUserBusiness();
			redisUtil.del("tableList");
			redisUtil.lSet("tableList", tableList);
			redisUtil.set(username+"_sessinId", request.getSession().getId());
			//把对象转变成一个json
			ObjectMapper mapper = new ObjectMapper();
			try {
				String userInfoObject = mapper.writeValueAsString(userInfo);
				HttpSession session = request.getSession();
				MySessionContext.AddSession(session);//将当前session存到静态map里
				// 当前登录用户
				session.setAttribute("userInfo", userInfoObject);
				// 当前登录用户id
				session.setAttribute("userId", userInfo.getUserId());
				// 当前登录用户机构id
				session.setAttribute("orgId", userInfo.getOrgId());
				// 当前组织信息
				OrgInfo orgInfo = orgInfoService.findById(userInfo.getOrgId());
				String orgInfoObject = mapper.writeValueAsString(orgInfo);
				session.setAttribute("orgInfo", orgInfoObject);
				// 当前用户所属的库信息及库编号
				OrgInfo depotInfo = orgInfoService.getDepotInfo(orgInfo);
				if(depotInfo!=null){
					String depotInfoObject = mapper.writeValueAsString(depotInfo);
					session.setAttribute("depotInfo", depotInfoObject);
					session.setAttribute("depotId", depotInfo.getOrgId());
				}
			} catch (JsonProcessingException e) {
				e.printStackTrace();
			}
			map.put("status", true);
			map.put("userInfo", userInfo);
		}
		return map;
	}

	@RequestMapping(value="/checkPwdBlack", method = RequestMethod.POST)
	@ApiOperation(value="查询密码字符串中是否包含密码黑名单里面的内容", notes = "密码可用就返回true")
	@ApiImplicitParams({
		@ApiImplicitParam(name = "newpwd", value = "密码", paramType = "form")
	})
	public boolean checkPwdBlack(String newpwd) {
		boolean flag = false;
		try {
			flag = userInfoService.checkPwdBlack(newpwd);
		} catch (Exception e) {
			e.printStackTrace();
		}
		return flag;
	}
}
