package com.chinaitop.depot.system.service.impl;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.annotation.Resource;

import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import com.alibaba.fastjson.JSONObject;
import com.chinaitop.depot.system.mapper.BasicEnumMapper;
import com.chinaitop.depot.system.mapper.OrgInfoMapper;
import com.chinaitop.depot.system.model.BasicEnum;
import com.chinaitop.depot.system.model.BasicEnumExample;
import com.chinaitop.depot.system.model.OrgInfo;
import com.chinaitop.depot.system.model.OrgInfoExample;
import com.chinaitop.depot.system.service.DataDockingService;
import com.chinaitop.depot.utils.ParameterUtil;

@Service
public class DataDockingServiceImpl implements DataDockingService {

	@Resource
	private OrgInfoMapper OrgInfoMapper;

	@Resource
	private BasicEnumMapper basicEnumMapper;

	@Override
	@Transactional(rollbackFor = Exception.class)
	public Map<String, String> saveOrUpdateSptOrgInfoData(String datas) throws Exception {
		Map<String, String> resultMap = new HashMap<>();
		
		JSONObject obj = null;
		if (StringUtils.isNotBlank(datas)) {
        	obj = JSONObject.parseObject(datas);
        } else {
        	resultMap.put("status", "error");
        	resultMap.put("msg", "所传的数据串是空的!");
        	return resultMap;
        }
		
		Map<String, Object> check = dataCheck(obj);
        String msg = check.get("msg").toString();
        if (msg.length() == 0) {
        	OrgInfo orgInfo = (OrgInfo) check.get("orgInfo");
        	OrgInfoMapper.updateByPrimaryKey(orgInfo);

        	resultMap.put("status", "success");
        	resultMap.put("msg", "操作成功");
		} else {
			resultMap.put("status", "error");
        	resultMap.put("msg", msg);
		}
		
		return resultMap;
	}

	/**
	 * 校验市平台传过来的组织机构数据
	 * @param obj
	 * @return
	 * @throws Exception
	 */
	private Map<String, Object> dataCheck(JSONObject obj) throws Exception {
		Map<String, Object> map = new HashMap<>();
	
		OrgInfo orgInfo = new OrgInfo();

		StringBuffer sbf = new StringBuffer(100);

		//库区标准编码
    	String kqdm = obj.getString("kqdm");
    	if (StringUtils.isNotBlank(kqdm)) {
    		OrgInfoExample example = new OrgInfoExample();
    		OrgInfoExample.Criteria criteria = example.createCriteria();
    		criteria.andKqbmEqualTo(kqdm.trim());
    		List<OrgInfo> orginfo_list = OrgInfoMapper.selectByExample(example);
        	if (null != orginfo_list && orginfo_list.size() > 0) {
        		orgInfo = orginfo_list.get(0);
        	} else {
        		map.put("status", "error");
    			map.put("msg", "kqdm有误或没有这个库区!");
    			return map;
        	}
		} else {
			map.put("status", "error");
			map.put("msg", "kqdm不能为空!");
        	return map;
		}

		//单位代码tyshxydm
    	String tyshxydm = obj.getString("tyshxydm");
    	if (StringUtils.isNotBlank(tyshxydm)) {
    		orgInfo.setCompanyOrgCode(tyshxydm.trim());
    		orgInfo.setCreditCode(tyshxydm.trim());
    	} else {
    		//sbf.append("tyshxydm不能为空;");
    	}

    	//单位名称
    	String qymc = obj.getString("qymc");
    	if (StringUtils.isNotBlank(qymc)) {
    		orgInfo.setOrgName(qymc.trim());
    		orgInfo.setShortName(qymc.trim());
    	} else {
    		//sbf.append("qymc不能为空;");
    	}

    	//单位类型dwlx
    	String dwlx = obj.getString("kqlb");
    	if (StringUtils.isNotBlank(dwlx)) {
    		String id = this.getBasicEnum(319, dwlx.trim(), "1");
    		if (StringUtils.isNotBlank(id)) {
    			orgInfo.setDepotClass(Integer.parseInt(id));
    		} else {
    			sbf.append("dwlx参数有误,请联系管理员;");
    		}
    	} else {
    		//sbf.append("dwlx不能为空;");
    	}

    	//注册日期zcrq，到时分秒
    	String zcrq = obj.getString("zcrq");
    	if (StringUtils.isNotBlank(zcrq)) {
    		orgInfo.setRegistDate(ParameterUtil.string2date(zcrq));
    	} else {
    		//sbf.append("zcrq不能为空;");
    	}

    	//注册资本zczb
    	String zczb = obj.getString("zczb");
    	if (StringUtils.isNotBlank(zczb)) {
    		orgInfo.setRegistCapital((int) Double.parseDouble(zczb.trim()));
    	}

    	//资产总额zcze
    	String zcze = obj.getString("zcze");
    	if (StringUtils.isNotBlank(zcze)) {
    		orgInfo.setTotalAssets((int) Double.parseDouble(zcze.trim()));
    	}

    	//法定代表人fddbr
    	String fddbr = obj.getString("fddbr");
    	if (StringUtils.isNotBlank(fddbr)) {
    		orgInfo.setLegalPerson(fddbr.trim());
    	} else {
    		//sbf.append("fddbr不能为空;");
    	}

    	//Y法人身份证号frsfzh
    	String frsfzh = obj.getString("frsfzh");
    	if (StringUtils.isNotBlank(frsfzh)) {
    		orgInfo.setCorporateCapacity(frsfzh.trim());
    	} else {
    		//sbf.append("frsfzh不能为空;");
    	}

    	//法人联系方式frsjh
    	String frsjh = obj.getString("frsjh");
    	if (StringUtils.isNotBlank(frsjh)) {
    		orgInfo.setFrlxfs(frsjh.trim());
    	}

    	//Y企业联系人qylxr
    	String qylxr = obj.getString("qylxr");
    	if (StringUtils.isNotBlank(qylxr)) {
    		orgInfo.setChargePerson(qylxr.trim());
    	} else {
    		//sbf.append("qylxr不能为空;");
    	}

    	//Y办公电话bgdh
    	String bgdh = obj.getString("bgdh");
    	if (StringUtils.isNotBlank(bgdh)) {
    		orgInfo.setOfficePhone(bgdh.trim());
    	} else {
    		//sbf.append("bgdh不能为空;");
    	}

    	//Y注册地址qygszcd
    	String qygszcd = obj.getString("qygszcd");
    	if (StringUtils.isNotBlank(qygszcd)) {
    		orgInfo.setRegisteredAddress(qygszcd.trim());
    	} else {
    		//sbf.append("qygszcd不能为空;");
    	}

    	//电子邮箱qydzyx
    	String qydzyx = obj.getString("qydzyx");
    	if (StringUtils.isNotBlank(qydzyx)) {
    		orgInfo.setMailBox(qydzyx.trim());
    	}

    	//企业官方网站地址(无)
    	//传真号码qyczhm
    	String qyczhm = obj.getString("qyczhm");
    	if (StringUtils.isNotBlank(qyczhm)) {
    		orgInfo.setFax(qyczhm.trim());
    	}

    	//邮政编码qyyb
    	String qyyb = obj.getString("qyyb");
    	if (StringUtils.isNotBlank(qyyb)) {
    		orgInfo.setPost(qyyb.trim());
    	}

    	//Y行政区划代码zccounty
    	String zccounty = obj.getString("zccounty");
    	if (StringUtils.isNotBlank(zccounty)) {
    		String id = this.getBasicEnum(1010, zccounty.trim(), "1");
    		if (StringUtils.isNotBlank(id)) {
    			orgInfo.setAreaCode(Integer.parseInt(id));
    		} else {
    			sbf.append("zccounty参数有误,请联系管理员;");
    		}
    		orgInfo.setRegisteredAddress(zccounty.trim());
    	} else {
    		//sbf.append("zccounty不能为空;");
    	}

    	//上级单位名称
    	//上级单位代码
    	//Y库区数kqsl
    	String kqsl = obj.getString("kqsl");
    	if (StringUtils.isNotBlank(kqsl)) {
    		orgInfo.setKqNumber(Integer.parseInt(kqsl.trim()));
    	} else {
    		//sbf.append("kqsl不能为空;");
    	}

    	//Y仓房数cfsl
    	String cfsl = obj.getString("cfsl");
    	if (StringUtils.isNotBlank(cfsl)) {
    		orgInfo.setHouseNumber(Integer.parseInt(cfsl.trim()));
    	} else {
    		//sbf.append("cfsl不能为空;");
    	}

    	//Y油罐数ygsl
    	String ygsl = obj.getString("ygsl");
    	if (StringUtils.isNotBlank(kqsl)) {
    		orgInfo.setTankNumber(Integer.parseInt(ygsl.trim()));
    	} else {
    		//sbf.append("ygsl不能为空;");
    	}

    	//经度qyjd
    	String qyjd = obj.getString("qyjd");
    	if (StringUtils.isNotBlank(qyjd)) {
    		orgInfo.setLongitude(qyjd.trim());
    	}

    	//纬度qywd
    	String qywd = obj.getString("qywd");
    	if (StringUtils.isNotBlank(qywd)) {
    		orgInfo.setLatitude(qywd.trim());
    	}

    	//Y库区产权kqcq
    	String kqcq = obj.getString("kqcq");
    	if (StringUtils.isNotBlank(kqcq)) {
    		orgInfo.setDepotProperty(kqcq.trim());
    	} else {
    		//sbf.append("kqcq不能为空;");
    	}

    	//Y有效仓容zcr
    	String zcr = obj.getString("zcr");
    	if (StringUtils.isNotBlank(zcr)) {
    		BigDecimal effectiveCapacity = new BigDecimal(zcr.trim()).setScale(2, BigDecimal.ROUND_DOWN);
    		orgInfo.setEffectiveCapacity(effectiveCapacity);
    	} else {
    		//sbf.append("zcr不能为空;");
    	}

    	//Y有效罐容gr
    	String gr = obj.getString("gr");
    	if (StringUtils.isNotBlank(gr)) {
    		orgInfo.setYxgr(Double.parseDouble(gr.trim()));
    	} else {
    		//sbf.append("gr不能为空;");
    	}

    	//Y占地面积zdmj
    	String zdmj = obj.getString("zdmj");
    	if (StringUtils.isNotBlank(zdmj)) {
    		BigDecimal depotArea = new BigDecimal(zdmj.trim()).setScale(2, BigDecimal.ROUND_DOWN);
    		orgInfo.setDepotArea(depotArea);
    	} else {
    		//sbf.append("zdmj不能为空;");
    	}

		String msg = sbf.toString();
		map.put("msg", msg);

		map.put("orgInfo", orgInfo);
		return map;
	}

	/**
	 * 通过父级ID和国标编码查询一条字典值的ID
	 * @param parentid
	 * @param gbcode
	 * @param dataType 数据类型：0是普通下拉框,1是树结构下拉框
	 * @return
	 * @throws Exception
	 */
	private String getBasicEnum(Integer parentid, String gbcode, String dataType) throws Exception {
		String id = "";
		
		if ("0".equals(dataType)) {
			BasicEnumExample example = new BasicEnumExample();
			BasicEnumExample.Criteria criteria = example.createCriteria();
			criteria.andParentidEqualTo(parentid);
			criteria.andGbcodeEqualTo(gbcode);
			criteria.andDelflagEqualTo(1);
			List<BasicEnum> list = basicEnumMapper.selectByExample(example);
			BasicEnum emm = null;
			if (null != list && list.size() > 0) {
				emm = list.get(0);
				id = emm.getEnumid().toString();
			}
		} else if ("1".equals(dataType)) {
			List<BasicEnum> resultList = queryAllChildren(parentid, "", gbcode);
			if (null != resultList && resultList.size() > 0) {
				for (int i =0; i < resultList.size(); i++) {
					BasicEnum e = resultList.get(i);
					if (e.getGbcode().equals(gbcode)) {
						id = e.getEnumid().toString();
						break;
					}
				}
			}
		}
		return id;
	}

	/**
	 * 递归查询一个数据结构
	 * @param enumId 树结构最上层节点ID
	 * @param planManage
	 * @return
	 * @throws Exception
	 */
	private List<BasicEnum> queryAllChildren(Integer enumId,String planManage, String gbcode) throws Exception {

		BasicEnumExample example = new BasicEnumExample();
		BasicEnumExample.Criteria criteria = example.createCriteria();
		criteria.andDelflagEqualTo(1);
		example.setOrderByClause("parentid asc, enumId asc");
		List<BasicEnum> list = basicEnumMapper.selectByExample(example); //查询到所有正常数据
		List<BasicEnum> firstlist = this.getChildren(list, enumId, planManage);
		List<BasicEnum> resultList = new ArrayList<>();
		resultList = this.getAllChildren(list, firstlist, resultList);

		return resultList;
	}

	private List<BasicEnum> getChildren(List<BasicEnum> list, Integer parentId,String planManage) {
		List<BasicEnum> children = new ArrayList<BasicEnum>();
		//计划管理下面的计划 粮食性质去掉储备粮的
		if(planManage!=null && planManage.equals("0")){
			for(BasicEnum be: list){
				//遍历出父id等于参数的id，add进子节点集合  除去储备粮=3045的和父节点是储备粮的
				if (be.getParentid().equals(parentId) && !be.getParentid().equals(3045) && !be.getEnumid().equals(3045)) {
					children.add(be);
				}
			}
		}else{
			for(BasicEnum be: list){
				//遍历出父id等于参数的id，add进子节点集合  
				if (be.getParentid().equals(parentId)) {
					children.add(be);
				}
			}
		}
        return children;
	}
	
	private List<BasicEnum> getAllChildren(List<BasicEnum> allList, 
			List<BasicEnum> childList, List<BasicEnum> resultList) {
        resultList.addAll(childList);
        for(BasicEnum be : childList) {
            List<BasicEnum> childList1 = this.getChildren(allList, be.getEnumid(),null);
            if(childList1.size()>0){
                this.getAllChildren(allList, childList1, resultList);
            }
        }
        return resultList;
	}
}
