package com.chinaitop.depot.system.service.impl;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.annotation.Resource;

import org.apache.commons.lang.StringUtils;
import org.springframework.stereotype.Service;

import com.chinaitop.depot.system.mapper.OrgInfoMapper;
import com.chinaitop.depot.system.model.OrgInfo;
import com.chinaitop.depot.system.model.OrgInfoExample;
import com.chinaitop.depot.system.service.OrgInfoService;

@Service
public class OrgInfoServiceImpl implements OrgInfoService {
	
	@Resource
	private OrgInfoMapper OrgInfoMapper;

	@Override
	public List<OrgInfo> queryByExample(OrgInfoExample example) {
		// TODO Auto-generated method stub
		return OrgInfoMapper.selectByExample(example);
	}

	@Override
	public OrgInfo findById(Integer orgId) {
		// TODO Auto-generated method stub
		return OrgInfoMapper.selectByPrimaryKey(orgId);
	}

	@Override
	public void add(OrgInfo orgInfo) {
		// TODO Auto-generated method stub
		OrgInfoMapper.insert(orgInfo);
	}

	@Override
	public void update(OrgInfo orgInfo) {
		// TODO Auto-generated method stub
		OrgInfoMapper.updateByPrimaryKey(orgInfo);
	}

	@Override
	public void remove(Integer orgId) {
		// TODO Auto-generated method stub
		OrgInfoMapper.deleteByPrimaryKey(orgId);
	}

	@Override
	public void updateSelective(OrgInfo orgInfo) {
		OrgInfoMapper.updateByPrimaryKeySelective(orgInfo);
	}

	@Override
	public List<OrgInfo> queryTreeByExample(OrgInfoExample example) {
		List<OrgInfo> orglist = OrgInfoMapper.selectByExample(example);
		List<OrgInfo> toplist = this.getChildren(orglist, 0);
		this.fillTree(orglist, toplist);
		return toplist;
	}

	/**
	 * 根据编号获取所有子节点
	 * @param list
	 * @param parentId
	 * @return
	 */
	private List<OrgInfo> getChildren(List<OrgInfo> list, Integer parentId) {
		List<OrgInfo> children = new ArrayList<OrgInfo>();
        for(OrgInfo org: list){
        	if (org.getParentId().equals(parentId)) {
        		children.add(org);
        	}
        }
        return children;
	}

	/**
	 * 递归填充树形结构
	 * @param allList
	 * @param childList
	 */
	public void fillTree(List<OrgInfo> allList, List<OrgInfo> childList) {
        if(childList!=null){
            for(OrgInfo org : childList) {
                List<OrgInfo> childList1 = this.getChildren(allList, org.getOrgId());
                fillTree(allList, childList1);
                org.setChildren(childList1);
            }
        }
    }

	@Override
	public OrgInfo getDepotInfo(OrgInfo orgInfo) {
		if (orgInfo.getOrgClassId()==5318) {
			return orgInfo;
		} else {
			if (orgInfo.getParentId()!=0) {
				OrgInfo parentOrg = OrgInfoMapper.selectByPrimaryKey(orgInfo.getParentId());
				return getDepotInfo(parentOrg);
			}
		}
		return null;
	}

	private void addChildren(List<OrgInfo> allList, List<OrgInfo> childList, List<OrgInfo> resultList) {
		resultList.addAll(childList);
		for(OrgInfo org : childList) {
			List<OrgInfo> childList1 = this.getChildren(allList, org.getOrgId());
			if(childList1.size()>0){
				addChildren(allList, childList1, resultList);
			}
		}
	}

	@Override
	public List<OrgInfo> getAllChildren(Integer orgId, String barnName) {
		OrgInfoExample example = new OrgInfoExample();
		OrgInfoExample.Criteria criteria = example.createCriteria();

		//传输过来的库点名称
		if(barnName!=""&&barnName!=null){
			String[] barn = barnName.split(":");
			if(StringUtils.isNotBlank(barn[0]) && barn[0].compareTo("null")!=0){
				criteria.andOrgNameEqualTo(barn[0]);
			}
			criteria.andOrgClassIdEqualTo(Integer.parseInt(barn[1]));
		}

		List<OrgInfo> orglist = OrgInfoMapper.selectByExample(example);
		OrgInfo orgInfo = OrgInfoMapper.selectByPrimaryKey(orgId);
		List<OrgInfo> toplist = this.getChildren(orglist, orgId);
		List<OrgInfo> resultList = new ArrayList<>();
		resultList.add(orgInfo);
		this.addChildren(orglist, toplist, resultList);
		return resultList;
	}
	
	@Override
	public List<OrgInfo> getTreeListByOrgId(Integer orgId) {
		// 当前库.
		OrgInfo rootOrgInfo = OrgInfoMapper.selectByPrimaryKey(orgId);
		
		/*this.fillTree(rootOrgInfo, toplist);
		
		OrgInfoExample example = new OrgInfoExample();
		Criteria createCriteria = example.createCriteria();
		createCriteria.andParentIdEqualTo(orgId);
		
		
		List<OrgInfo> toplist = this.getChildren(orglist, 0);
		this.fillTree(orglist, toplist);
		return toplist;*/
		
		return null;
	}

	@Override
	public List<OrgInfo> getOrgLevel(Map<String,Object> map) {
		return OrgInfoMapper.getOrgLevel(map);
	}
	
	@Override
	public void updateOrgLevel(OrgInfo orgInfo) {
		Map<String,Object> map = new HashMap<String,Object>();
		if(orgInfo.getParentId() == 0 && (orgInfo.getLevelCode() == null || ("").equals(orgInfo.getLevelCode()))) {//根机构
			map.put("parentId", orgInfo.getOrgId());
			map.put("parentLevel", "10");
		}else {
			map.put("parentId", orgInfo.getOrgId());
			map.put("parentLevel", orgInfo.getLevelCode());
		}
		OrgInfoMapper.updateOrgLevel(map);
	}
	
	@Override
	public List<OrgInfo> getAllDepotInfo() {
		return OrgInfoMapper.getAllDepotInfo();
	}

}
