package com.chinaitop.depot.system.service.impl;

import com.chinaitop.depot.system.mapper.FuncInfoMapper;
import com.chinaitop.depot.system.mapper.RoleFuncMapper;
import com.chinaitop.depot.system.mapper.UserRoleMapper;
import com.chinaitop.depot.system.model.*;
import com.chinaitop.depot.system.service.RoleFuncService;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.List;

@Service
public class RoleFuncServiceImpl implements RoleFuncService {
	
	@Resource
	private RoleFuncMapper roleFuncMapper;
	
	@Resource
	private UserRoleMapper userRoleMapper;
	
	@Resource
	private FuncInfoMapper funcInfoMapper;

	@Override
	public List<RoleFunc> queryByRoleId(Integer roleId) {
		// TODO Auto-generated method stub
		RoleFuncExample example = new RoleFuncExample();
		example.createCriteria().andRoleIdEqualTo(roleId);
		return roleFuncMapper.selectByExample(example);
	}

	@Override
	public void save(Integer roleId, String[] funcIds) {
		// 先删除此角色所有数据
		RoleFuncExample example = new RoleFuncExample();
		example.createCriteria().andRoleIdEqualTo(roleId);
		roleFuncMapper.deleteByExample(example);
		// 新增
		for (String funcId : funcIds) {
			RoleFunc roleFunc = new RoleFunc();
			roleFunc.setRoleId(roleId);
			roleFunc.setFuncId(Integer.parseInt(funcId));
			roleFuncMapper.insert(roleFunc);
		}
	}
	
	@Override
	public void saveForList(Integer roleId, List<FuncInfo> funcList) {
		// 先删除此角色所有数据
		RoleFuncExample example = new RoleFuncExample();
		example.createCriteria().andRoleIdEqualTo(roleId);
		roleFuncMapper.deleteByExample(example);
		// 新增
		for (FuncInfo func : funcList) {
			RoleFunc roleFunc = new RoleFunc();
			roleFunc.setRoleId(roleId);
			roleFunc.setFuncId(func.getFuncId());
			roleFuncMapper.insert(roleFunc);
		}
	}

	@Override
	public List<FuncInfo> queryFuncByUserId(Integer userId) {
		List<FuncInfo> funclist = new ArrayList<>();
		UserRoleExample example = new UserRoleExample();
		example.createCriteria().andUserIdEqualTo(userId);
		List<UserRole> urList = userRoleMapper.selectByExample(example);
		if (urList.size() > 0) {
			List<Integer> roleIds = new ArrayList<>();
			for (UserRole userRole : urList) {
				roleIds.add(userRole.getRoleId());
			}
			RoleFuncExample roleFuncExample = new RoleFuncExample();
			roleFuncExample.createCriteria().andRoleIdIn(roleIds);
			List<RoleFunc> rfList = roleFuncMapper.selectByExample(roleFuncExample);
			if (rfList.size() > 0) {
				List<Integer> funcIds = new ArrayList<>();
				for (RoleFunc roleFunc : rfList) {
					funcIds.add(roleFunc.getFuncId());
				}
				FuncInfoExample funcInfoExample = new FuncInfoExample();
				funcInfoExample.createCriteria().andFuncIdIn(funcIds);
				funclist = funcInfoMapper.selectByExample(funcInfoExample);
			}
		}
		return funclist;
	}

	@Override
	public void remove(Integer roleId) {
		RoleFuncExample example = new RoleFuncExample();
		example.createCriteria().andRoleIdEqualTo(roleId);
		roleFuncMapper.deleteByExample(example);
	}

}
