package com.chinaitop.depot.system.service.impl;

import java.math.BigInteger;
import java.security.MessageDigest;
import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.annotation.Resource;

import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import com.chinaitop.depot.system.mapper.UserBusinessMapper;
import com.chinaitop.depot.system.mapper.UserInfoMapper;
import com.chinaitop.depot.system.mapper.UserRoleMapper;
import com.chinaitop.depot.system.model.UserInfo;
import com.chinaitop.depot.system.model.UserInfoExample;
import com.chinaitop.depot.system.model.UserInfoExample.Criteria;
import com.chinaitop.depot.system.model.UserRole;
import com.chinaitop.depot.system.model.UserRoleExample;
import com.chinaitop.depot.system.service.UserInfoService;
import com.chinaitop.depot.system.service.UserOrgService;
import com.chinaitop.depot.utils.NetStateUtil;
import com.chinaitop.depot.utils.SHA1Util;
import com.chinaitop.depot.webservice.ExchangeServer;
import com.chinaitop.depot.webservice.IExchangeService;
import com.fasterxml.jackson.databind.ObjectMapper;

@Service
public class UserInfoServiceImpl implements UserInfoService {

	@Resource
	private UserInfoMapper UserInfoMapper;

	@Resource
	private UserRoleMapper userRoleMapper;

	@Resource
	private UserBusinessMapper userBusinessMapper;

	@Resource
	private UserOrgService userOrgService;

	@Override
	public UserInfo valid(String UserInfoName, String password) {
		// 获取MD5加密后的字符串
		//password = this.getMD5(password);
		UserInfoExample example = new UserInfoExample();
		example.createCriteria().andUsernameEqualTo(UserInfoName).andPasswordEqualTo(password);
		List<UserInfo> list = UserInfoMapper.selectByExample(example);
		if (list.size() > 0) {
			UserInfo userInfo = list.get(0);
			UserRoleExample urExample = new UserRoleExample();
			urExample.createCriteria().andUserIdEqualTo(userInfo.getUserId());
			List<UserRole> urList = userRoleMapper.selectByExample(urExample);
			List<String> roleIds = new ArrayList<>();
			for (UserRole userRole : urList) {
				roleIds.add(String.valueOf(userRole.getRoleId()));
			}
			userInfo.setRoleIds(roleIds);
			return userInfo;
		}
		return null;
	}

	@Override
	public List<UserInfo> queryByExample(UserInfoExample example) {
		// TODO Auto-generated method stub
		return UserInfoMapper.selectByExample(example);
	}

	@Override
	public UserInfo findById(Integer userId) {
		// TODO Auto-generated method stub
		UserInfo userInfo =  UserInfoMapper.selectByPrimaryKey(userId);
		UserRoleExample example = new UserRoleExample();
		example.createCriteria().andUserIdEqualTo(userInfo.getUserId());
		List<UserRole> urList = userRoleMapper.selectByExample(example);
		List<String> roleIds = new ArrayList<>();
		for (UserRole userRole : urList) {
			roleIds.add(String.valueOf(userRole.getRoleId()));
		}
		userInfo.setRoleIds(roleIds);
		return userInfo;
	}

	@Override
	public void add(UserInfo userInfo) {
		// TODO Auto-generated method stub
		//userInfo.setPassword(this.getMD5("123456"));
		//把密码再用SHA1加密一次
		String sha1_pwd = SHA1Util.encode(userInfo.getPassword());
		userInfo.setPassword(sha1_pwd);
		userInfo.setCreateTime(new Date());
		userInfo.setUpdateTime(userInfo.getCreateTime());
		UserInfoMapper.insert(userInfo);
		// 添加用户角色信息
		for (String roleId : userInfo.getRoleIds()) {
			UserRole userRole = new UserRole();
			userRole.setUserId(userInfo.getUserId());
			userRole.setRoleId(Integer.parseInt(roleId));
			userRoleMapper.insert(userRole);
		}
	}

	@Override
	public void update(UserInfo userInfo) {
		// TODO Auto-generated method stub
		UserInfoMapper.updateByPrimaryKey(userInfo);
		// 更新用户角色信息,先删除，再增加
		/*UserRoleExample userRoleExample = new UserRoleExample();
		userRoleExample.createCriteria().andUserIdEqualTo(userInfo.getUserId());
		userRoleMapper.deleteByExample(userRoleExample);
		for (String roleId : userInfo.getRoleIds()) {
			UserRole userRole = new UserRole();
			userRole.setUserId(userInfo.getUserId());
			userRole.setRoleId(Integer.parseInt(roleId));
			userRoleMapper.insert(userRole);
		}*/
	}

	@Override
	public void remove(Integer userId) {
		// TODO Auto-generated method stub
		UserInfoMapper.deleteByPrimaryKey(userId);
	}

	@Override
	public boolean validUsername(Integer userId, String username) {
		// TODO Auto-generated method stub
		UserInfoExample example = new UserInfoExample();
		Criteria criteria = example.createCriteria();
		if (userId != null) {
			criteria.andUserIdNotEqualTo(userId);
		}
		criteria.andUsernameEqualTo(username);
		List<UserInfo> list = UserInfoMapper.selectByExampleNoDatePermission(example);
		if (list.size() > 0) {
			return false;
		}
		return true;
	}
	
	@Override
	public String validPhone(String phoneNumber, Integer userId) {
		UserInfoExample example = new UserInfoExample();
		Criteria criteria = example.createCriteria();
		criteria.andMobileEqualTo(phoneNumber);
		if (userId != null) {
			criteria.andUserIdNotEqualTo(userId);
		}
		List<UserInfo> list = UserInfoMapper.selectByExampleNoDatePermission(example);
		if (list.size() > 0) {
			return list.get(0).getUsername();
		}
		return "";
	}
	
	@Override
	public String queryPhoneByusername(String username) {
		UserInfoExample example = new UserInfoExample();
		Criteria criteria = example.createCriteria();
		criteria.andUsernameEqualTo(username);
		List<UserInfo> list = UserInfoMapper.selectByExample(example);
		if (list.size() > 0) {
			return list.get(0).getMobile();
		}
		return "";
	}

	@Override
	public boolean validPassword(String resPassword, String oldPassword) {
		oldPassword = this.getMD5(oldPassword);//md5加密
		String sha1_pwd = SHA1Util.encode(oldPassword);//sha1加密
		return resPassword.equals(sha1_pwd);
	}

    @Override
    public void resetPassword(Integer userId,String userPwd) {
	    UserInfo userInfo = UserInfoMapper.selectByPrimaryKey(userId);
		//再用SHA1加密一次
		String sha1_pwd = SHA1Util.encode(userPwd);
	    userInfo.setPassword(sha1_pwd);
	    userInfo.setUpdateTime(new Date());
	    UserInfoMapper.updateByPrimaryKey(userInfo);
    }

	@Override
	public List<UserInfo> queryAll(HashMap<Object, Object> map) {
		// TODO Auto-generated method stub
		return UserInfoMapper.queryAll(map);
	}

	/**
	 * 对字符串md5加密(小写+字母)
	 *
	 * @param str 传入要加密的字符串
	 * @return  MD5加密后的字符串
	 */
	@Override
	public String getMD5(String str) {
		try {
			// 生成一个MD5加密计算摘要
			MessageDigest md = MessageDigest.getInstance("MD5");
			// 计算md5函数
			md.update(str.getBytes());
			// digest()最后确定返回md5 hash值，返回值为8为字符串。因为md5 hash值是16位的hex值，实际上就是8位的字符
			// BigInteger函数则将8位的字符串转换成16位hex值，用字符串来表示；得到字符串形式的hash值
			return new BigInteger(1, md.digest()).toString(16);
		} catch (Exception e) {
			e.printStackTrace();
			return null;
		}
	}

	@Override
	public List<String> queryUserBusiness() {
		return userBusinessMapper.queryBusinessList();
	}

	/**
	 * 查找审批用户信息
	 * @param map
	 * @return
	 */
	@Override
	public List<Map<Object, Object>> queryAuditUser(Map<Object, Object> map) {
		return UserInfoMapper.queryAuditUser(map);
	}
	
	@Override
	public List<UserInfo> queryAllByOrgClass(Map<Object, Object> map) {
		return UserInfoMapper.queryAllByOrgClass(map);
	}

	@Override
	@Transactional
	public void deleteAll(Integer userId) throws Exception {

		UserInfoMapper.deleteByPrimaryKey(userId);//删除当前人
		userRoleMapper.deleteById(userId);//删除用户角色信息
		userOrgService.deleteById(userId);//删除用户机构权限信息

		//删除用对应的员工信息
		/* 查询当前用户对应的员工信息ID */
		List<Map<String, Object>> list = UserInfoMapper.findKeeper(userId);

		if (null != list && list.size() > 0) {
			/* 删除当前用户对应的员工所保管的仓房信息 */
			//Map<String, Object> map = null;
			for (Map<String, Object> map : list) {
				Integer id = Integer.parseInt(map.get("id")+"");
				UserInfoMapper.deleteKeeperHouseObj(id);
			}

			/* 删除当前用户对应的员工信息 */
			UserInfoMapper.deleteKeeperObj(userId);
		}

	}

	@Override
	@Transactional(rollbackFor = Exception.class)
	public Map<String, Object> save(String userInfoJson, String userRoleIds,
									String crkRoleIds, String wsdlUrl,
									String FLATID, String KEY) throws Exception {
		Map<String, Object> modelMap = new HashMap<>();

		//Pattern pattern = Pattern.compile("^[-\\+]?[\\d]*$");
		// JSON字符串转对象
		ObjectMapper mapper = new ObjectMapper();
		UserInfo userInfo = (UserInfo)mapper.readValue(userInfoJson, UserInfo.class);
		if(userInfo.getUserId() == null){
			this.add(userInfo);
		}else{
			Integer userid = userInfo.getUserId();
			UserInfo newuser = UserInfoMapper.selectByPrimaryKey(userid);
			//谷歌浏览器在修改当前登录用户的时候传到后台的密码是明文，所以这个地方用原来的密码重新赋值一下，避免修改用户信息后密码成了明文
			userInfo.setPassword(newuser.getPassword());
			userInfo.setUpdateTime(new Date());
			this.update(userInfo);
			userRoleMapper.deleteById(userInfo.getUserId());
		}

		UserRole userRole = new UserRole();
		if (StringUtils.isNotBlank(userRoleIds) && !"[]".equals(userRoleIds)) {
			userRoleIds = userRoleIds.substring(1, userRoleIds.length());
			userRoleIds = userRoleIds.substring(0, userRoleIds.length() - 1);
			String[] idArray = userRoleIds.split(",");
			userRole.setUserId(userInfo.getUserId());
			for (String id : idArray) {
				userRole.setRoleId(Integer.parseInt(id));
				userRoleMapper.insertSelective(userRole);
			}
		}

		if (StringUtils.isNotBlank(crkRoleIds)) {//保存出入库用户角色信息
			if("正常".equals(NetStateUtil.connectingAddress(wsdlUrl))) {
				try {
					String json = "{userId:\""+userInfo.getUserId()+"\", roleIds:"+crkRoleIds+"}";
					ExchangeServer wb = new ExchangeServer();
					IExchangeService action = wb.getExchangeServiceImplPort();
					action.saveUserRoles(FLATID, KEY, json);
				} catch (Exception e) {
					e.printStackTrace();
				}
			}

		}

		modelMap.put("status", "success");
		return modelMap;
	}

	@Override
	public String checkRepeat(Integer orgId, String sfzhm, Integer userId, String type) throws Exception {
		String flag = "false";
		UserInfoExample example = new UserInfoExample();
		Criteria criteria = example.createCriteria();
		if (null != orgId) {
			criteria.andOrgIdEqualTo(orgId);
		}

		if (StringUtils.isNotBlank(sfzhm)) {
			criteria.andSfzhmEqualTo(sfzhm);
		}
		
		if (null != userId && "update".equals(type)) {
			criteria.andUserIdNotEqualTo(userId);
		}

		List<UserInfo> list =  queryByExample(example);
		if (null == list || list.size() == 0) {
			flag = "true";
		}
		return flag;
	}

	@Override
	public List<Map<Object, Object>> sptgetUserInfo(Map<Object, Object> map) {
		// TODO Auto-generated method stub
		return UserInfoMapper.sptgetUserInfo(map);
	}

	@Override
	public boolean checkPwdBlack(String pwd) throws Exception {
		boolean flag = false;
		List<String> list = UserInfoMapper.selectPwdBlack(pwd);
		if (null == list || list.size() == 0) {
			flag = true;
		}
		return flag;
	}

}
